@@ -1,5 +1,5 @@
 /* cat -- concatenate files and print on the standard output.
-   Copyright (C) 1988, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1988, 1990, 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,26 +12,17 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program; if not, write to the Free Software Foundation,
+   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Differences from the Unix cat:
    * Always unbuffered, -u is ignored.
-   * 100 times faster with -v -u.
-   * 20 times faster with -v.
+   * Usually much faster than other versions of cat, the difference
+   is especially apparent when using the -v option.
 
    By tege@sics.se, Torbjorn Granlund, advised by rms, Richard Stallman. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 #include <stdio.h>
 #include <getopt.h>
@@ -41,12 +32,16 @@
 #endif
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
+/* Undefine, to avoid warning about redefinition on some systems.  */
+#undef max
 #define max(h,i) ((h) > (i) ? (h) : (i))
 
 char *stpcpy ();
 char *xmalloc ();
-void error ();
+int full_write ();
+int safe_read ();
 
 static void cat ();
 static void next_line_num ();
@@ -69,7 +64,7 @@ static char line_buf[13] =
 {' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', '0', '\t', '\0'};
 
 /* Position in `line_buf' where printing starts.  This will not change
-   unless the number of lines are more than 999999.  */
+   unless the number of lines is larger than 999999.  */
 static char *line_num_print = line_buf + 5;
 
 /* Position of the first digit in `line_buf'.  */
@@ -89,15 +84,16 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION] [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Concatenate FILE(s), or standard input, to standard output.\n\
 \n\
   -b, --number-nonblank    number nonblank output lines\n\
   -e                       equivalent to -vE\n\
@@ -113,7 +109,7 @@ Usage: %s [OPTION] [FILE]...\n\
       --version            output version information and exit\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
-");
+"));
     }
   exit (status);
 }
@@ -259,7 +255,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("cat - %s\n", version_string);
       exit (0);
     }
 
@@ -271,7 +267,7 @@ main (argc, argv)
   /* Get device, i-node number, and optimal blocksize of output.  */
 
   if (fstat (output_desc, &stat_buf) < 0)
-    error (1, errno, "standard output");
+    error (1, errno, _("standard output"));
 
   outsize = ST_BLKSIZE (stat_buf);
   /* Input file can be output file for non-regular files.
@@ -286,7 +282,13 @@ main (argc, argv)
       out_ino = stat_buf.st_ino;
     }
   else
-    check_redirection = 0;
+    {
+      check_redirection = 0;
+#ifdef lint  /* Suppress `used before initialized' warning.  */
+      out_dev = 0;
+      out_ino = 0;
+#endif
+    }
 
   /* Check if any of the input files are the same as the output file.  */
 
@@ -332,7 +334,7 @@ main (argc, argv)
       if (check_redirection
 	  && stat_buf.st_dev == out_dev && stat_buf.st_ino == out_ino)
 	{
-	  error (0, 0, "%s: input file is output file", infile);
+	  error (0, 0, _("%s: input file is output file"), infile);
 	  exit_stat = 1;
 	  goto contin;
 	}
@@ -391,7 +393,7 @@ main (argc, argv)
   if (have_read_stdin && close (0) < 0)
     error (1, errno, "-");
   if (close (1) < 0)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
 
   exit (exit_stat);
 }
@@ -417,7 +419,7 @@ simple_cat (buf, bufsize)
     {
       /* Read a block of input.  */
 
-      n_read = read (input_desc, buf, bufsize);
+      n_read = safe_read (input_desc, buf, bufsize);
       if (n_read < 0)
 	{
 	  error (0, errno, "%s", infile);
@@ -432,8 +434,8 @@ simple_cat (buf, bufsize)
 
       /* Write this block out.  */
 
-      if (write (output_desc, buf, n_read) != n_read)
-	error (1, errno, "write error");
+      if (full_write (output_desc, buf, n_read) < 0)
+	error (1, errno, _("write error"));
     }
 }
 
@@ -516,8 +518,8 @@ cat (inbuf, insize, outbuf, outsize, quote,
 	      unsigned char *wp = outbuf;
 	      do
 		{
-		  if (write (output_desc, wp, outsize) != outsize)
-		    error (1, errno, "write error");
+		  if (full_write (output_desc, wp, outsize) < 0)
+		    error (1, errno, _("write error"));
 		  wp += outsize;
 		}
 	      while (bpout - wp >= outsize);
@@ -525,7 +527,7 @@ cat (inbuf, insize, outbuf, outsize, quote,
 	      /* Move the remaining bytes to the beginning of the
 		 buffer.  */
 
-	      bcopy (wp, outbuf, bpout - wp);
+	      memmove (outbuf, wp, bpout - wp);
 	      bpout = outbuf + (bpout - wp);
 	    }
 
@@ -547,13 +549,18 @@ cat (inbuf, insize, outbuf, outsize, quote,
 		     HP-UX returns ENOTTY on pipes.
 		     SunOS returns EINVAL and
 		     More/BSD returns ENODEV on special files
-		     like /dev/null.  */
+		     like /dev/null.
+		     Irix-5 returns ENOSYS on pipes.  */
 		  if (errno == EOPNOTSUPP || errno == ENOTTY
-		      || errno == EINVAL || errno == ENODEV)
+		      || errno == EINVAL || errno == ENODEV
+#ifdef ENOSYS
+		      || errno == ENOSYS
+#endif
+		      )
 		    use_fionread = 0;
 		  else
 		    {
-		      error (0, errno, "cannot do ioctl on `%s'", infile);
+		      error (0, errno, _("cannot do ioctl on `%s'"), infile);
 		      exit_stat = 1;
 		      newlines2 = newlines;
 		      return;
@@ -564,14 +571,14 @@ cat (inbuf, insize, outbuf, outsize, quote,
 		{
 		  int n_write = bpout - outbuf;
 
-		  if (write (output_desc, outbuf, n_write) != n_write)
-		    error (1, errno, "write error");
+		  if (full_write (output_desc, outbuf, n_write) < 0)
+		    error (1, errno, _("write error"));
 		  bpout = outbuf;
 		}
 
 	      /* Read more input into INBUF.  */
 
-	      n_read = read (input_desc, inbuf, insize);
+	      n_read = safe_read (input_desc, inbuf, insize);
 	      if (n_read < 0)
 		{
 		  error (0, errno, "%s", infile);
