@@ -1,5 +1,5 @@
 /* csplit - split a file into sections determined by context lines
-   Copyright (C) 1991 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Stuart Kemp, cpsrk@groper.jcu.edu.au.
    Modified by David MacKenzie, djm@gnu.ai.mit.edu. */
@@ -30,6 +30,7 @@
 #include "regex.h"
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
 #ifdef STDC_HEADERS
 #include <stdlib.h>
@@ -42,10 +43,8 @@ char *realloc ();
 #define MAX(a,b) (((a) > (b)) ? (a) : (b))
 #endif
 
-void error ();
 int safe_read ();
 
-
 static char *xrealloc ();
 static char *xmalloc ();
 static void cleanup ();
@@ -74,7 +73,7 @@ struct control
   struct re_pattern_buffer re_compiled;	/* Compiled regular expression. */
   int offset;			/* Offset from regexp to split at. */
   int lines_required;		/* Number of lines required. */
-  int repeat;			/* Repeat count. */
+  unsigned int repeat;		/* Repeat count. */
   int repeat_forever;		/* Non-zero if `*' used as a repeat count. */
   int argnum;			/* ARGV index. */
   boolean ignore;		/* If true, produce no output (for regexp). */
@@ -161,9 +160,6 @@ static unsigned last_line_number = 0;
 /* Number of the line currently being examined. */
 static unsigned current_line = 0;
 
-/* Number of the last line in the input file. */
-static unsigned last_line_in_file = 0;
-
 /* If TRUE, we have read EOF. */
 static boolean have_read_eof = FALSE;
 
@@ -180,10 +176,10 @@ static char *suffix = NULL;
 static int digits = 2;
 
 /* Number of files created so far. */
-static unsigned files_created = 0;
+static unsigned int files_created = 0;
 
 /* Number of bytes written to current file. */
-static unsigned bytes_written;
+static unsigned int bytes_written;
 
 /* Output file pointer. */
 static FILE *output_stream = NULL;
@@ -208,7 +204,7 @@ static boolean elide_empty_files;
 static struct control *controls;
 
 /* Number of elements in `controls'. */
-static unsigned control_used;
+static unsigned int control_used;
 
 /* If non-zero, display usage information and exit.  */
 static int show_help;
@@ -234,14 +230,14 @@ static struct option const longopts[] =
 
 static char *
 xmalloc (n)
-     unsigned n;
+     unsigned int n;
 {
   char *p;
 
   p = malloc (n);
   if (p == NULL)
     {
-      error (0, 0, "virtual memory exhausted");
+      error (0, 0, _("virtual memory exhausted"));
       cleanup ();
     }
   return p;
@@ -255,7 +251,7 @@ xmalloc (n)
 static char *
 xrealloc (p, n)
      char *p;
-     unsigned n;
+     unsigned int n;
 {
   if (p == NULL)
     return xmalloc (n);
@@ -267,7 +263,7 @@ xrealloc (p, n)
   p = realloc (p, n);
   if (p == NULL)
     {
-      error (0, 0, "virtual memory exhausted");
+      error (0, 0, _("virtual memory exhausted"));
       cleanup ();
     }
   return p;
@@ -284,33 +280,33 @@ xrealloc (p, n)
 static void
 save_to_hold_area (start, num)
      char *start;
-     unsigned num;
+     unsigned int num;
 {
   hold_area = start;
   hold_count = num;
 }
 
-/* Read up to MAX chars from the input stream into DEST.
+/* Read up to MAX_N_BYTES chars from the input stream into DEST.
    Return the number of chars read. */
 
 static int
-read_input (dest, max)
+read_input (dest, max_n_bytes)
      char *dest;
-     unsigned max;
+     unsigned int max_n_bytes;
 {
   int bytes_read;
 
-  if (max == 0)
+  if (max_n_bytes == 0)
     return 0;
 
-  bytes_read = safe_read (input_desc, dest, max);
+  bytes_read = safe_read (input_desc, dest, max_n_bytes);
 
   if (bytes_read == 0)
     have_read_eof = TRUE;
 
   if (bytes_read < 0)
     {
-      error (0, errno, "read error");
+      error (0, errno, _("read error"));
       cleanup ();
     }
 
@@ -394,15 +390,15 @@ keep_new_line (b, line_start, line_len)
    a pointer is kept to this area, which will be used when
    the next buffer is filled. */
 
-static unsigned
+static unsigned int
 record_line_starts (b)
      struct buffer_record *b;
 {
   char *line_start;		/* Start of current line. */
   char *line_end;		/* End of each line found. */
-  unsigned bytes_left;		/* Length of incomplete last line. */
-  unsigned lines;		/* Number of lines found. */
-  unsigned line_length;		/* Length of each line found. */
+  unsigned int bytes_left;	/* Length of incomplete last line. */
+  unsigned int lines;		/* Number of lines found. */
+  unsigned int line_length;	/* Length of each line found. */
 
   if (b->bytes_used == 0)
     return 0;
@@ -430,7 +426,6 @@ record_line_starts (b)
 	{
 	  keep_new_line (b, line_start, bytes_left);
 	  lines++;
-	  last_line_in_file = last_line_number + lines;
 	}
       else
 	save_to_hold_area (line_start, bytes_left);
@@ -448,7 +443,7 @@ record_line_starts (b)
 
 static struct buffer_record *
 create_new_buffer (size)
-     unsigned size;
+     unsigned int size;
 {
   struct buffer_record *new_buffer;
 
@@ -468,11 +463,11 @@ create_new_buffer (size)
 
 static struct buffer_record *
 get_new_buffer (min_size)
-     unsigned min_size;
+     unsigned int min_size;
 {
   struct buffer_record *p, *q;
   struct buffer_record *new_buffer; /* Buffer to return. */
-  unsigned alloc_size;		/* Actual size that will be requested. */
+  unsigned int alloc_size;	/* Actual size that will be requested. */
 
   alloc_size = START_SIZE;
   while (min_size > alloc_size)
@@ -564,9 +559,9 @@ static boolean
 load_buffer ()
 {
   struct buffer_record *b;
-  unsigned bytes_wanted = START_SIZE; /* Minimum buffer size. */
-  unsigned bytes_avail;		/* Size of new buffer created. */
-  unsigned lines_found;		/* Number of lines in this new buffer. */
+  unsigned int bytes_wanted = START_SIZE; /* Minimum buffer size. */
+  unsigned int bytes_avail;		/* Size of new buffer created. */
+  unsigned int lines_found;		/* Number of lines in this new buffer. */
   char *p;			/* Place to load into buffer. */
 
   if (have_read_eof)
@@ -587,14 +582,14 @@ load_buffer ()
       if (hold_count)
 	{
 	  if (p != hold_area)
-	    bcopy (hold_area, p, hold_count);
+	    memcpy (p, hold_area, hold_count);
 	  p += hold_count;
 	  b->bytes_used += hold_count;
 	  bytes_avail -= hold_count;
 	  hold_count = 0;
 	}
 
-      b->bytes_used += (unsigned) read_input (p, bytes_avail);
+      b->bytes_used += (unsigned int) read_input (p, bytes_avail);
 
       lines_found = record_line_starts (b);
       bytes_wanted = b->bytes_alloc * 2;
@@ -611,11 +606,11 @@ load_buffer ()
 
 /* Return the line number of the first line that has not yet been retrieved. */
 
-static unsigned
+static unsigned int
 get_first_line_in_buffer ()
 {
   if (head == NULL && !load_buffer ())
-    error (1, errno, "input disappeared");
+    error (1, errno, _("input disappeared"));
 
   return head->first_available;
 }
@@ -664,7 +659,7 @@ remove_line ()
 
 static struct cstring *
 find_line (linenum)
-     unsigned linenum;
+     unsigned int linenum;
 {
   struct buffer_record *b;
 
@@ -680,7 +675,7 @@ find_line (linenum)
 	{
 	  /* The line is in this buffer. */
 	  struct line *l;
-	  unsigned offset;	/* How far into the buffer the line is. */
+	  unsigned int offset;	/* How far into the buffer the line is. */
 
 	  l = b->line_start;
 	  offset = linenum - b->start_line;
@@ -729,20 +724,20 @@ set_input_file (name)
 
 static void
 write_to_file (last_line, ignore, argnum)
-     unsigned last_line;
+     unsigned int last_line;
      boolean ignore;
      int argnum;
 {
   struct cstring *line;
-  unsigned first_line;		/* First available input line. */
-  unsigned lines;		/* Number of lines to output. */
-  unsigned i;
+  unsigned int first_line;		/* First available input line. */
+  unsigned int lines;		/* Number of lines to output. */
+  unsigned int i;
 
   first_line = get_first_line_in_buffer ();
 
   if (first_line > last_line)
     {
-      error (0, 0, "%s: line number out of range", global_argv[argnum]);
+      error (0, 0, _("%s: line number out of range"), global_argv[argnum]);
       cleanup ();
     }
 
@@ -753,7 +748,7 @@ write_to_file (last_line, ignore, argnum)
       line = remove_line ();
       if (line == NULL)
 	{
-	  error (0, 0, "%s: line number out of range", global_argv[argnum]);
+	  error (0, 0, _("%s: line number out of range"), global_argv[argnum]);
 	  cleanup ();
 	}
       if (!ignore)
@@ -780,10 +775,10 @@ handle_line_error (p, repetition)
      struct control *p;
      int repetition;
 {
-  fprintf (stderr, "%s: `%d': line number out of range",
+  fprintf (stderr, _("%s: `%d': line number out of range"),
 	   program_name, p->lines_required);
   if (repetition)
-    fprintf (stderr, " on repetition %d\n", repetition);
+    fprintf (stderr, _(" on repetition %d\n"), repetition);
   else
     fprintf (stderr, "\n");
 
@@ -800,8 +795,8 @@ process_line_count (p, repetition)
      struct control *p;
      int repetition;
 {
-  unsigned linenum;
-  unsigned last_line_to_save = p->lines_required * (repetition + 1);
+  unsigned int linenum;
+  unsigned int last_line_to_save = p->lines_required * (repetition + 1);
   struct cstring *line;
 
   create_output_file ();
@@ -835,11 +830,11 @@ regexp_error (p, repetition, ignore)
      int repetition;
      boolean ignore;
 {
-  fprintf (stderr, "%s: `%s': match not found",
+  fprintf (stderr, _("%s: `%s': match not found"),
 	   program_name, global_argv[p->argnum]);
 
   if (repetition)
-    fprintf (stderr, " on repetition %d\n", repetition);
+    fprintf (stderr, _(" on repetition %d\n"), repetition);
   else
     fprintf (stderr, "\n");
 
@@ -861,8 +856,8 @@ process_regexp (p, repetition)
      int repetition;
 {
   struct cstring *line;		/* From input file. */
-  unsigned line_len;	/* To make "$" in regexps work. */
-  unsigned break_line;		/* First line number of next file. */
+  unsigned int line_len;	/* To make "$" in regexps work. */
+  unsigned int break_line;	/* First line number of next file. */
   boolean ignore = p->ignore;	/* If TRUE, skip this section. */
   int ret;
 
@@ -898,7 +893,7 @@ process_regexp (p, repetition)
 			   0, line_len, (struct re_registers *) 0);
 	  if (ret == -2)
 	    {
-	      error (0, 0, "error in regular expression search");
+	      error (0, 0, _("error in regular expression search"));
 	      cleanup ();
 	    }
 	  if (ret == -1)
@@ -938,7 +933,7 @@ process_regexp (p, repetition)
 			   0, line_len, (struct re_registers *) 0);
 	  if (ret == -2)
 	    {
-	      error (0, 0, "error in regular expression search");
+	      error (0, 0, _("error in regular expression search"));
 	      cleanup ();
 	    }
 	  if (ret >= 0)
@@ -962,7 +957,7 @@ process_regexp (p, repetition)
 static void
 split_file ()
 {
-  int i, j;
+  unsigned int i, j;
 
   for (i = 0; i < control_used; i++)
     {
@@ -989,7 +984,7 @@ split_file ()
 
 static char *
 make_filename (num)
-     int num;
+     unsigned int num;
 {
   strcpy (filename_space, prefix);
   if (suffix)
@@ -1020,7 +1015,7 @@ create_output_file ()
 static void
 delete_all_files ()
 {
-  int i;
+  unsigned int i;
   char *name;
 
   for (i = 0; i < files_created; i++)
@@ -1041,7 +1036,7 @@ close_output_file ()
     {
       if (fclose (output_stream) == EOF)
 	{
-	  error (0, errno, "write error for `%s'", output_filename);
+	  error (0, errno, _("write error for `%s'"), output_filename);
 	  cleanup ();
 	}
       if (bytes_written == 0 && elide_empty_files)
@@ -1115,6 +1110,7 @@ new_control_record ()
 /* Convert string NUM to an integer and put the value in *RESULT.
    Return a TRUE if the string consists entirely of digits,
    FALSE if not. */
+/* FIXME: use xstrtoul in place of this function.  */
 
 static boolean
 string_to_number (result, num)
@@ -1150,10 +1146,10 @@ check_for_offset (p, str, num)
      char *num;
 {
   if (*num != '-' && *num != '+')
-    error (1, 0, "%s: `+' or `-' expected after delimeter", str);
+    error (1, 0, _("%s: `+' or `-' expected after delimeter"), str);
 
   if (!string_to_number (&p->offset, num + 1))
-    error (1, 0, "%s: integer expected after `%c'", str, *num);
+    error (1, 0, _("%s: integer expected after `%c'"), str, *num);
 
   if (*num == '-')
     p->offset = -p->offset;
@@ -1174,14 +1170,14 @@ parse_repeat_count (argnum, p, str)
 
   end = str + strlen (str) - 1;
   if (*end != '}')
-    error (1, 0, "%s: `}' is required in repeat count", str);
+    error (1, 0, _("%s: `}' is required in repeat count"), str);
   *end = '\0';
 
   if (str+1 == end-1 && *(str+1) == '*')
     p->repeat_forever = 1;
   else
     if (!string_to_number (&p->repeat, str +  1))
-      error (1, 0, "%s}: integer required between `{' and `}'",
+      error (1, 0, _("%s}: integer required between `{' and `}'"),
 	     global_argv[argnum]);
 
   *end = '}';
@@ -1205,9 +1201,9 @@ extract_regexp (argnum, ignore, str)
   struct control *p;
   const char *err;
 
-  closing_delim = rindex (str + 1, delim);
+  closing_delim = strrchr (str + 1, delim);
   if (closing_delim == NULL)
-    error (1, 0, "%s: closing delimeter `%c' missing", str, delim);
+    error (1, 0, _("%s: closing delimeter `%c' missing"), str, delim);
 
   len = closing_delim - str - 1;
   p = new_control_record ();
@@ -1223,7 +1219,7 @@ extract_regexp (argnum, ignore, str)
   err = re_compile_pattern (p->regexpr, len, &p->re_compiled);
   if (err)
     {
-      error (0, 0, "%s: invalid regular expression: %s", str, err);
+      error (0, 0, _("%s: invalid regular expression: %s"), str, err);
       cleanup ();
     }
 
@@ -1256,7 +1252,7 @@ parse_patterns (argc, start, argv)
 	  p = new_control_record ();
 	  p->argnum = i;
 	  if (!string_to_number (&p->lines_required, argv[i]))
-	    error (1, 0, "%s: invalid pattern", argv[i]);
+	    error (1, 0, _("%s: invalid pattern"), argv[i]);
 	}
 
       if (i + 1 < argc && *argv[i + 1] == '{')
@@ -1307,7 +1303,7 @@ get_format_width (format_ptr)
 
   start = *format_ptr;
   for (; **format_ptr; (*format_ptr)++)
-    if (**format_ptr < '0' || **format_ptr > '9')
+    if (!ISDIGIT (**format_ptr))
       break;
 
   ch_save = **format_ptr;
@@ -1346,7 +1342,7 @@ get_format_prec (format_ptr)
 
   start = *format_ptr;
   for (; **format_ptr; (*format_ptr)++)
-    if (**format_ptr < '0' || **format_ptr > '9')
+    if (!ISDIGIT (**format_ptr))
       break;
 
   /* ANSI 4.9.6.1 says that if the precision is negative, it's as good as
@@ -1379,14 +1375,14 @@ get_format_conv_type (format_ptr)
       break;
 
     case 0:
-      error (1, 0, "missing conversion specifier in suffix");
+      error (1, 0, _("missing conversion specifier in suffix"));
       break;
 
     default:
       if (ISPRINT (ch))
-        error (1, 0, "invalid conversion specifier in suffix: %c", ch);
+        error (1, 0, _("invalid conversion specifier in suffix: %c"), ch);
       else
-	error (1, 0, "invalid conversion specifier in suffix: \\%.3o", ch);
+	error (1, 0, _("invalid conversion specifier in suffix: \\%.3o"), ch);
     }
 }
 
@@ -1418,17 +1414,18 @@ max_out (format)
     }
 
   if (percents == 0)
-    error (1, 0, "missing %% conversion specification in suffix");
+    error (1, 0, _("missing %% conversion specification in suffix"));
   else if (percents > 1)
-    error (1, 0, "too many %% conversion specifications in suffix");
+    error (1, 0, _("too many %% conversion specifications in suffix"));
 
   return out_count;
 }
 
 static void
-interrupt_handler ()
+interrupt_handler (signum)
+     int signum;
 {
-  error (0, 0, "interrupted");
+  error (0, 0, _("interrupted"));
   cleanup ();
 }
 
@@ -1502,7 +1499,7 @@ main (argc, argv)
 
       case 'n':
 	if (!string_to_number (&digits, optarg))
-	  error (1, 0, "%s: invalid number", optarg);
+	  error (1, 0, _("%s: invalid number"), optarg);
 	break;
 
       case 's':
@@ -1527,8 +1524,11 @@ main (argc, argv)
   if (show_help)
     usage (0);
 
-  if (optind >= argc - 1)
-    usage (1);
+  if (argc - optind < 2)
+    {
+      error (0, 0, _("too few arguments"));
+      usage (1);
+    }
 
   if (suffix)
     filename_space = (char *) xmalloc (strlen (prefix) + max_out (suffix) + 2);
@@ -1543,7 +1543,7 @@ main (argc, argv)
 
   if (close (input_desc) < 0)
     {
-      error (0, errno, "read error");
+      error (0, errno, _("read error"));
       cleanup ();
     }
 
@@ -1555,15 +1555,17 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... FILE PATTERN...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Output pieces of FILE separated by PATTERN(s) to files `xx01', `xx02', ...,\n\
+and output byte counts of each piece to standard output.\n\
 \n\
   -b, --suffix-format=FORMAT use sprintf FORMAT instead of %%d\n\
   -f, --prefix=PREFIX        use PREFIX instead of `xx'\n\
@@ -1583,7 +1585,7 @@ Read standard input if FILE is -.  Each PATTERN may be:\n\
   {*}                repeat the previous pattern as many times as possible\n\
 \n\
 A line OFFSET is a required `+' or `-' followed by a positive integer.\n\
-");
+"));
     }
   exit (status);
 }
