@@ -73,6 +73,7 @@
 #include <sys/types.h>
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
 #define FATAL_ERROR(s)							\
   do									\
@@ -109,7 +110,6 @@ struct range_pair
 
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 
 /* This buffer is used to support the semantics of the -s option
    (or lack of same) when the specified field list includes (does
@@ -164,7 +164,7 @@ static enum operating_mode operating_mode;
 static int suppress_non_delimited;
 
 /* The delimeter character for field mode. */
-static unsigned char delim;
+static int delim;
 
 /* Nonzero if we have ever read standard input. */
 static int have_read_stdin;
@@ -188,19 +188,19 @@ static struct option const longopts[] =
 };
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Print selected parts of lines from each FILE to standard output.\n\
 \n\
   -b, --bytes=LIST        output only these bytes\n\
   -c, --characters=LIST   output only these characters\n\
@@ -220,7 +220,7 @@ range, or many ranges separated by commas.  Each range is one of:\n\
   -M    from first to M'th (included) byte, character or field\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
-");
+"));
     }
   exit (status);
 }
@@ -242,11 +242,7 @@ With no FILE, or when FILE is -, read standard input.\n\
    including the null terminator), or -1 on error or EOF.  */
 
 static int
-getstr (lineptr, n, stream, terminator)
-     char **lineptr;
-     int *n;
-     FILE *stream;
-     char terminator;
+getstr (char **lineptr, int *n, FILE *stream, char terminator)
 {
   int nchars_avail;		/* Allocated but unused chars in *LINEPTR.  */
   char *read_pos;		/* Where we're reading into *LINEPTR. */
@@ -313,10 +309,9 @@ getstr (lineptr, n, stream, terminator)
 }
 
 static int
-print_kth (k)
-     int k;
+print_kth (int k)
 {
-  return ((eol_range_start > 0 && eol_range_start <= k)
+  return ((0 < eol_range_start && eol_range_start <= k)
 	  || (k <= max_range_endpoint && printable_field[k]));
 }
 
@@ -337,8 +332,7 @@ print_kth (k)
    one bit per field index instead of a whole `int' per index.  */
 
 static int
-set_fields (fieldstr)
-     const char *fieldstr;
+set_fields (const char *fieldstr)
 {
   int initial = 1;		/* Value of first number in a range.  */
   int dash_found = 0;		/* Nonzero if a '-' is found in this field.  */
@@ -349,7 +343,7 @@ set_fields (fieldstr)
   struct range_pair *rp;
   unsigned int n_rp;
   unsigned int n_rp_allocated;
-  int i;
+  unsigned int i;
 
   n_rp = 0;
   n_rp_allocated = 16;
@@ -364,7 +358,7 @@ set_fields (fieldstr)
 	{
 	  /* Starting a range. */
 	  if (dash_found)
-	    FATAL_ERROR ("invalid byte or field list");
+	    FATAL_ERROR (_("invalid byte or field list"));
 	  dash_found++;
 	  fieldstr++;
 
@@ -395,7 +389,7 @@ set_fields (fieldstr)
 		{
 		  /* `m-n' or `-n' (1-n). */
 		  if (value < initial)
-		    FATAL_ERROR ("invalid byte or field list");
+		    FATAL_ERROR (_("invalid byte or field list"));
 
 		  /* Is there already a range going to end of line? */
 		  if (eol_range_start != 0)
@@ -455,7 +449,7 @@ set_fields (fieldstr)
 	  fieldstr++;
 	}
       else
-	FATAL_ERROR ("invalid byte or field list");
+	FATAL_ERROR (_("invalid byte or field list"));
     }
 
   max_range_endpoint = 0;
@@ -491,8 +485,7 @@ set_fields (fieldstr)
 /* Read from stream STREAM, printing to standard output any selected bytes.  */
 
 static void
-cut_bytes (stream)
-     FILE *stream;
+cut_bytes (FILE *stream)
 {
   int byte_idx;			/* Number of chars in the line so far. */
 
@@ -528,8 +521,7 @@ cut_bytes (stream)
 /* Read from stream STREAM, printing to standard output any selected fields.  */
 
 static void
-cut_fields (stream)
-     FILE *stream;
+cut_fields (FILE *stream)
 {
   int c;
   int field_idx;
@@ -631,8 +623,7 @@ cut_fields (stream)
 }
 
 static void
-cut_stream (stream)
-     FILE *stream;
+cut_stream (FILE *stream)
 {
   if (operating_mode == byte_mode)
     cut_bytes (stream);
@@ -644,8 +635,7 @@ cut_stream (stream)
    Return 0 if successful, 1 if not. */
 
 static int
-cut_file (file)
-     char *file;
+cut_file (char *file)
 {
   FILE *stream;
 
@@ -682,9 +672,7 @@ cut_file (file)
 }
 
 void
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int optc, exit_status = 0;
 
@@ -710,26 +698,26 @@ main (argc, argv)
 	case 'c':
 	  /* Build the byte list. */
 	  if (operating_mode != undefined_mode)
-	    FATAL_ERROR ("only one type of list may be specified");
+	    FATAL_ERROR (_("only one type of list may be specified"));
 	  operating_mode = byte_mode;
 	  if (set_fields (optarg) == 0)
-	    FATAL_ERROR ("missing list of positions");
+	    FATAL_ERROR (_("missing list of positions"));
 	  break;
 
 	case 'f':
 	  /* Build the field list. */
 	  if (operating_mode != undefined_mode)
-	    FATAL_ERROR ("only one type of list may be specified");
+	    FATAL_ERROR (_("only one type of list may be specified"));
 	  operating_mode = field_mode;
 	  if (set_fields (optarg) == 0)
-	    FATAL_ERROR ("missing list of fields");
+	    FATAL_ERROR (_("missing list of fields"));
 	  break;
 
 	case 'd':
 	  /* New delimiter. */
 	  /* Interpret -d '' to mean `use the NUL byte as the delimiter.'  */
 	  if (optarg[0] != '\0' && optarg[1] != '\0')
-	    FATAL_ERROR ("the delimiter must be a single character");
+	    FATAL_ERROR (_("the delimiter must be a single character"));
 	  delim = optarg[0];
 	  break;
 
@@ -755,14 +743,14 @@ main (argc, argv)
     usage (0);
 
   if (operating_mode == undefined_mode)
-    FATAL_ERROR ("you must specify a list of bytes, characters, or fields");
+    FATAL_ERROR (_("you must specify a list of bytes, characters, or fields"));
 
   if (delim != '\0' && operating_mode != field_mode)
-    FATAL_ERROR ("a delimiter may be specified only when operating on fields");
+    FATAL_ERROR (_("a delimiter may be specified only when operating on fields"));
 
   if (suppress_non_delimited && operating_mode != field_mode)
-    FATAL_ERROR ("suppressing non-delimited lines makes sense\n\
-\tonly when operating on fields");
+    FATAL_ERROR (_("suppressing non-delimited lines makes sense\n\
+\tonly when operating on fields"));
 
   if (delim == '\0')
     delim = '\t';
@@ -779,7 +767,7 @@ main (argc, argv)
       exit_status = 1;
     }
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
 
   exit (exit_status);
 }
