@@ -1,5 +1,5 @@
 /* expand - convert tabs to spaces
-   Copyright (C) 1989, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* By default, convert all tabs to spaces.
    Preserves backspace characters in the output; they decrement the
@@ -43,6 +43,7 @@
 #include <sys/types.h>
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
 /* The number of bytes added at a time to the amount of memory
    allocated for the output line. */
@@ -54,7 +55,6 @@
 
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 
 static FILE *next_file ();
 static void add_tabstop ();
@@ -185,7 +185,7 @@ main (argc, argv)
   if (have_read_stdin && fclose (stdin) == EOF)
     error (1, errno, "-");
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
 
   exit (exit_status);
 }
@@ -213,7 +213,7 @@ parse_tabstops (stops)
 	  tabval = tabval * 10 + *stops - '0';
 	}
       else
-	error (1, 0, "tab size contains an invalid character");
+	error (1, 0, _("tab size contains an invalid character"));
     }
 
   add_tabstop (tabval);
@@ -229,7 +229,8 @@ add_tabstop (tabval)
   if (tabval == -1)
     return;
   if (first_free_tab % TABLIST_BLOCK == 0)
-    tab_list = (int *) xrealloc (tab_list, first_free_tab + TABLIST_BLOCK);
+    tab_list = (int *) xrealloc (tab_list, first_free_tab
+				 + TABLIST_BLOCK * sizeof (tab_list[0]));
   tab_list[first_free_tab++] = tabval;
 }
 
@@ -247,9 +248,9 @@ validate_tabstops (tabs, entries)
   for (i = 0; i < entries; i++)
     {
       if (tabs[i] == 0)
-	error (1, 0, "tab size cannot be 0");
+	error (1, 0, _("tab size cannot be 0"));
       if (tabs[i] <= prev_tab)
-	error (1, 0, "tab sizes must be ascending");
+	error (1, 0, _("tab sizes must be ascending"));
       prev_tab = tabs[i];
     }
 }
@@ -388,15 +389,17 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Convert tabs in each FILE to spaces, writing to standard output.\n\
+With no FILE, or when FILE is -, read standard input.\n\
 \n\
   -i, --initial       do not convert TABs after non whitespace\n\
   -t, --tabs=NUMBER   have tabs NUMBER characters apart, not 8\n\
@@ -404,9 +407,8 @@ Usage: %s [OPTION]... [FILE]...\n\
       --help          display this help and exit\n\
       --version       output version information and exit\n\
 \n\
-Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.  With\n\
-no FILE, or when FILE is -, read standard input.\n\
-");
+Instead of -t NUMBER or -t LIST, -NUMBER or -LIST may be used.\n\
+"));
     }
   exit (status);
 }
