@@ -1,5 +1,5 @@
 /* GNU fmt -- simple text formatter.
-   Copyright (C) 1994 Free Software Foundation, Inc.
+   Copyright (C) 1994, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,22 +13,23 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
 /* Written by Ross Paterson <rap@doc.ic.ac.uk>.  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <stdio.h>
-
 #include <sys/types.h>
-#include "system.h"
+#include <getopt.h>
 
-#include "getopt.h"
+/* Redefine.  Otherwise, systems (Unicos for one) with headers that define
+   it to be a type get syntax errors for the variable declaration below.  */
+#define word unused_word_type
+
+#include "system.h"
 #include "version.h"
+#include "error.h"
 
 /* The following parameters represent the program's idea of what is
    "best".  Adjust to taste, subject to the caveats given.  */
@@ -141,36 +142,32 @@ struct Word
 
 /* Forward declarations.  */
 
-/* My AC_PROTOTYPES would be better than __STDC__.  FIXME :-).  */
-#if __STDC__
-#define	_(x) x
-#else
-#define	_(x) ()
+/* My fp_PROTOTYPES would be better than __STDC__.  FIXME :-).  */
+#ifndef __P
+# if __STDC__
+#  define __P(Args) Args
+# else
+#  define __P(Args) ()
+# endif
 #endif
 
-#ifdef HAVE_VPRINTF
-void error _ ((int, int, const char *,...));
-#else
-void error ();
-#endif
-
-static void set_prefix _ ((char *p));
-static void fmt _ ((FILE *f));
-static bool get_paragraph _ ((FILE *f));
-static int get_line _ ((FILE *f, int c));
-static int get_prefix _ ((FILE *f));
-static int get_space _ ((FILE *f, int c));
-static int copy_rest _ ((FILE *f, int c));
-static bool same_para _ ((int c));
-static void flush_paragraph _ ((void));
-static void fmt_paragraph _ ((void));
-static void check_punctuation _ ((WORD *w));
-static COST base_cost _ ((WORD *this));
-static COST line_cost _ ((WORD *next, int len));
-static void put_paragraph _ ((WORD *finish));
-static void put_line _ ((WORD *w, int indent));
-static void put_word _ ((WORD *w));
-static void put_space _ ((int space));
+static void set_prefix __P ((char *p));
+static void fmt __P ((FILE *f));
+static bool get_paragraph __P ((FILE *f));
+static int get_line __P ((FILE *f, int c));
+static int get_prefix __P ((FILE *f));
+static int get_space __P ((FILE *f, int c));
+static int copy_rest __P ((FILE *f, int c));
+static bool same_para __P ((int c));
+static void flush_paragraph __P ((void));
+static void fmt_paragraph __P ((void));
+static void check_punctuation __P ((WORD *w));
+static COST base_cost __P ((WORD *this));
+static COST line_cost __P ((WORD *next, int len));
+static void put_paragraph __P ((WORD *finish));
+static void put_line __P ((WORD *w, int indent));
+static void put_word __P ((WORD *w));
+static void put_space __P ((int space));
 
 /* The name this program was run with.  */
 const char *program_name;
@@ -278,13 +275,16 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n", program_name);
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
   else
     {
-      printf ("Usage: %s [-DIGITS] [OPTION]... [FILE]...\n", program_name);
-      fputs ("\
-Mandatory arguments to long options are mandatory for short options too.\n\
+      printf (_("Usage: %s [-DIGITS] [OPTION]... [FILE]...\n"), program_name);
+      fputs (_("\
+Reformat each paragraph in the FILE(s), writing to standard output.\n\
+If no FILE or if FILE is `-', read standard input.\n\
 \n\
+Mandatory arguments to long options are mandatory for short options too.\n\
   -c, --crown-margin        preserve indentation of first two lines\n\
   -s, --split-only          split long lines, but do not refill\n\
   -t, --tagged-paragraph    indentation of first line different from second\n\
@@ -294,8 +294,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
       --help                display this help and exit\n\
       --version             output version information and exit\n\
 \n\
-In -wNUMBER, the letter `w' may be omitted.  Input FILEs are concatenated.\n\
-If no FILE or if FILE is `-', standard input is read.\n",
+In -wNUMBER, the letter `w' may be omitted.\n"),
 	     stdout);
     }
   exit (status);
@@ -387,7 +386,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("fmt - %s\n", version_string);
       exit (0);
     }
 
@@ -554,7 +553,7 @@ get_paragraph (f)
   return TRUE;
 }
 
-/* Copy a line which failed to match the prefix to the output, or which
+/* Copy to the output a line that failed to match the prefix, or that
    was blank after the prefix.  In the former case, C is the character
    that failed to match the prefix.  In the latter, C is \n or EOF.
    Return the character (\n or EOF) ending the line.  */
@@ -567,7 +566,7 @@ copy_rest (f, c)
   register const char *s;
 
   out_column = 0;
-  if (in_column > next_prefix_indent || (c != '\n' && c != EOF))
+  if (in_column > next_prefix_indent && c != '\n' && c != EOF)
     {
       put_space (next_prefix_indent);
       for (s = prefix; out_column != in_column; out_column++)
@@ -765,10 +764,10 @@ flush_paragraph ()
     }
   put_paragraph (split_point);
 
-  /* Copy text of words down to start of parabuf -- we use bcopy because
+  /* Copy text of words down to start of parabuf -- we use memmove because
      the source and target may overlap.  */
 
-  bcopy (split_point->text, parabuf, (size_t) (wptr - split_point->text));
+  memmove (parabuf, split_point->text, (size_t) (wptr - split_point->text));
   shift = split_point->text - parabuf;
   wptr -= shift;
 
@@ -777,10 +776,10 @@ flush_paragraph ()
   for (w = split_point; w <= word_limit; w++)
     w->text -= shift;
 
-  /* Copy words from split_point down to word -- we use bcopy because
+  /* Copy words from split_point down to word -- we use memmove because
      the source and target may overlap.  */
 
-  bcopy ((char *) split_point, (char *) word,
+  memmove ((char *) word, (char *) split_point,
 	 (word_limit - split_point + 1) * sizeof (WORD));
   word_limit -= split_point - word;
 }
