@@ -1,5 +1,5 @@
 /* head -- output first part of file(s)
-   Copyright (C) 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,18 +13,9 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
-/* Options:
-   -b			Print first N 512-byte blocks.
-   -c, --bytes=N[bkm]	Print first N bytes
-			[or 512-byte blocks, kilobytes, or megabytes].
-   -k			Print first N kilobytes.
-   -N, -l, -n, --lines=N	Print first N lines.
-   -m			Print first N megabytes.
-   -q, --quiet, --silent	Never print filename headers.
-   -v, --verbose		Always print filename headers.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
+/* Options: (see usage)
    Reads from standard input if no files are given or when a filename of
    ``-'' is encountered.
    By default, filename headers are printed only if more than one file
@@ -40,6 +31,7 @@
 #include <sys/types.h>
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
 /* Number of lines/chars/blocks to head. */
 #define DEFAULT_NUMBER 10
@@ -60,7 +52,6 @@ enum header_mode
   multiple_files, always, never
 };
 
-void error ();
 int safe_read ();
 
 static int head ();
@@ -151,7 +142,7 @@ main (argc, argv)
 	      break;
 
 	    default:
-	      error (0, 0, "unrecognized option `-%c'", *argv[1]);
+	      error (0, 0, _("unrecognized option `-%c'"), *argv[1]);
 	      usage (1);
 	    }
 	  ++argv[1];
@@ -179,7 +170,7 @@ main (argc, argv)
 	getnum:
 	  number = atou (optarg);
 	  if (number == -1)
-	    error (1, 0, "invalid number `%s'", optarg);
+	    error (1, 0, _("invalid number `%s'"), optarg);
 	  break;
 
 	case 'q':
@@ -223,7 +214,7 @@ main (argc, argv)
   if (have_read_stdin && close (0) < 0)
     error (1, errno, "-");
   if (fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
 
   exit (exit_status);
 }
@@ -238,7 +229,7 @@ head_file (filename, number)
   if (!strcmp (filename, "-"))
     {
       have_read_stdin = 1;
-      filename = "standard input";
+      filename = _("standard input");
       if (print_headers)
 	write_header (filename);
       return head (filename, 0, number);
@@ -305,7 +296,7 @@ head_bytes (filename, fd, bytes_to_write)
       if (bytes_read > bytes_to_write)
 	bytes_read = bytes_to_write;
       if (fwrite (buffer, 1, bytes_read, stdout) == 0)
-	error (1, errno, "write error");
+	error (1, errno, _("write error"));
       bytes_to_write -= bytes_read;
     }
   return 0;
@@ -336,7 +327,7 @@ head_lines (filename, fd, lines_to_write)
 	if (buffer[bytes_to_write++] == '\n' && --lines_to_write == 0)
 	  break;
       if (fwrite (buffer, 1, bytes_to_write, stdout) == 0)
-	error (1, errno, "write error");
+	error (1, errno, _("write error"));
     }
   return 0;
 }
@@ -386,18 +377,21 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Print first 10 lines of each FILE to standard output.\n\
+With more than one FILE, precede each with a header giving the file name.\n\
+With no FILE, or when FILE is -, read standard input.\n\
 \n\
   -c, --bytes=SIZE         print first SIZE bytes\n\
-  -l, -n, --lines=NUMBER   print first NUMBER lines instead of first 10\n\
+  -n, --lines=NUMBER   print first NUMBER lines instead of first 10\n\
   -q, --quiet, --silent    never print headers giving file names\n\
   -v, --verbose            always print headers giving file names\n\
       --help               display this help and exit\n\
@@ -405,9 +399,8 @@ Usage: %s [OPTION]... [FILE]...\n\
 \n\
 SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
 If -VALUE is used as first OPTION, read -c VALUE when one of\n\
-multipliers bkm follows concatenated, else read -n VALUE.  With no\n\
-FILE, or when FILE is -, read standard input.\n\
-");
+multipliers bkm follows concatenated, else read -n VALUE.\n\
+"));
     }
   exit (status);
 }
