@@ -1,5 +1,5 @@
 /* join - join lines of two files on a common field
-   Copyright (C) 1991 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written by Mike Haertel, mike@gnu.ai.mit.edu. */
 
@@ -28,48 +28,54 @@
 #include "system.h"
 #include "version.h"
 #include "long-options.h"
+#include "error.h"
+
+#define join system_join
 
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 static void usage ();
 
+/* Undefine, to avoid warning about redefinition on some systems.  */
+#undef min
+#undef max
 #define min(A, B) ((A) < (B) ? (A) : (B))
 #define max(A, B) ((A) > (B) ? (A) : (B))
 
 /* An element of the list describing the format of each
    output line. */
 struct outlist
-{
-  int file;			/* File to take field from (1 or 2). */
-  int field;			/* Field number to print. */
-  struct outlist *next;
-};
+  {
+    int file;			/* File to take field from (1 or 2). */
+    int field;			/* Field number to print. */
+    struct outlist *next;
+  };
 
 /* A field of a line. */
 struct field
-{
-  char *beg;			/* First character in field. */
-  char *lim;			/* Character after last character in field. */
-};
+  {
+    const char *beg;		/* First character in field. */
+    size_t len;			/* The length of the field. */
+  };
 
 /* A line read from an input file.  Newlines are not stored. */
 struct line
-{
-  char *beg;			/* First character in line. */
-  char *lim;			/* Character after last character in line. */
-  int nfields;			/* Number of elements in `fields'. */
-  struct field *fields;
-};
+  {
+    char *beg;			/* First character in line. */
+    char *lim;			/* Character after last character in line. */
+    int nfields;		/* Number of elements in `fields'. */
+    int nfields_allocated;	/* Number of elements in `fields'. */
+    struct field *fields;
+  };
 
 /* One or more consecutive lines read from a file that all have the
    same join field value. */
 struct seq
-{
-  int count;			/* Elements used in `lines'. */
-  int alloc;			/* Elements allocated in `lines'. */
-  struct line *lines;
-};
+  {
+    int count;			/* Elements used in `lines'. */
+    int alloc;			/* Elements allocated in `lines'. */
+    struct line *lines;
+  };
 
 /* The name this program was run with. */
 char *program_name;
@@ -108,8 +114,25 @@ static struct option const longopts[] =
 };
 
 /* Used to print non-joining lines */
-static struct line blank1;
-static struct line blank2;
+static struct line uni_blank;
+
+static void
+ADD_FIELD (line, field, len)
+     struct line *line;
+     const char *field;
+     size_t len;
+{
+  if (line->nfields >= line->nfields_allocated)
+    {
+      line->nfields_allocated = (3 * line->nfields_allocated) / 2 + 1;
+      line->fields = (struct field *) xrealloc ((char *) line->fields,
+						(line->nfields_allocated
+						 * sizeof (struct field)));
+    }
+  line->fields[line->nfields].beg = field;
+  line->fields[line->nfields].len = len;
+  ++(line->nfields);
+}
 
 /* Fill in the `fields' structure in LINE. */
 
@@ -117,44 +140,44 @@ static void
 xfields (line)
      struct line *line;
 {
-  static int nfields = 2;
   int i;
   register char *ptr, *lim;
 
-  line->fields = (struct field *) xmalloc (nfields * sizeof (struct field));
-
   ptr = line->beg;
   lim = line->lim;
 
   for (i = 0; ptr < lim; ++i)
     {
-      if (i == nfields)
-	{
-	  nfields *= 2;
-	  line->fields = (struct field *)
-	    xrealloc ((char *) line->fields, nfields * sizeof (struct field));
-	}
       if (tab)
 	{
-	  line->fields[i].beg = ptr;
+	  char *beg;
+
+	  beg = ptr;
 	  while (ptr < lim && *ptr != tab)
 	    ++ptr;
-	  line->fields[i].lim = ptr;
+	  ADD_FIELD (line, beg, ptr - beg);
 	  if (ptr < lim)
 	    ++ptr;
 	}
       else
 	{
-	  line->fields[i].beg = ptr;
+	  char *beg;
+
+	  beg = ptr;
 	  while (ptr < lim && !ISSPACE (*ptr))
 	    ++ptr;
-	  line->fields[i].lim = ptr;
+	  ADD_FIELD (line, beg, ptr - beg);
 	  while (ptr < lim && ISSPACE (*ptr))
 	    ++ptr;
 	}
     }
 
-  line->nfields = i;
+  if (ptr > line->beg && ((tab && ISSPACE (ptr[-1])) || ptr[-1] == tab))
+    {
+      /* Add one more (empty) field because the last character of the
+	 line was a delimiter.  */
+      ADD_FIELD (line, NULL, 0);
+    }
 }
 
 /* Read a line from FP into LINE and split it into fields.
@@ -192,6 +215,9 @@ get_line (fp, line)
 
   line->beg = ptr;
   line->lim = line->beg + i;
+  line->nfields_allocated = 0;
+  line->nfields = 0;
+  line->fields = NULL;
   xfields (line);
   return 1;
 }
@@ -202,6 +228,7 @@ freeline (line)
 {
   free ((char *) line->fields);
   free (line->beg);
+  line->beg = NULL;
 }
 
 static void
@@ -239,6 +266,10 @@ static void
 delseq (seq)
      struct seq *seq;
 {
+  int i;
+  for (i = 0; i < seq->count; i++)
+    if (seq->lines[i].beg)
+      freeline (&seq->lines[i]);
   free ((char *) seq->lines);
 }
 
@@ -250,15 +281,14 @@ keycmp (line1, line2)
      struct line *line1;
      struct line *line2;
 {
-  char *beg1, *beg2;		/* Start of field to compare in each file. */
+  const char *beg1, *beg2;	/* Start of field to compare in each file. */
   int len1, len2;		/* Length of fields to compare. */
   int diff;
 
   if (join_field_1 < line1->nfields)
     {
       beg1 = line1->fields[join_field_1].beg;
-      len1 = line1->fields[join_field_1].lim
-	- line1->fields[join_field_1].beg;
+      len1 = line1->fields[join_field_1].len;
     }
   else
     {
@@ -269,8 +299,7 @@ keycmp (line1, line2)
   if (join_field_2 < line2->nfields)
     {
       beg2 = line2->fields[join_field_2].beg;
-      len2 = line2->fields[join_field_2].lim
-	- line2->fields[join_field_2].beg;
+      len2 = line2->fields[join_field_2].len;
     }
   else
     {
@@ -300,7 +329,7 @@ prfield (n, line)
 
   if (n < line->nfields)
     {
-      len = line->fields[n].lim - line->fields[n].beg;
+      len = line->fields[n].len;
       if (len)
 	fwrite (line->fields[n].beg, 1, len, stdout);
       else if (empty_filler)
@@ -333,6 +362,13 @@ prjoin (line1, line2)
     {
       int i;
 
+      if (line1 == &uni_blank)
+	{
+	  struct line *t;
+	  t = line1;
+	  line1 = line2;
+	  line2 = t;
+	}
       prfield (join_field_1, line1);
       for (i = 0; i < join_field_1 && i < line1->nfields; ++i)
 	{
@@ -382,7 +418,7 @@ join (fp1, fp2)
       if (diff < 0)
 	{
 	  if (print_unpairables_1)
-	    prjoin (&seq1.lines[0], &blank2);
+	    prjoin (&seq1.lines[0], &uni_blank);
 	  freeline (&seq1.lines[0]);
 	  seq1.count = 0;
 	  getseq (fp1, &seq1);
@@ -391,7 +427,7 @@ join (fp1, fp2)
       if (diff > 0)
 	{
 	  if (print_unpairables_2)
-	    prjoin (&blank1, &seq2.lines[0]);
+	    prjoin (&uni_blank, &seq2.lines[0]);
 	  freeline (&seq2.lines[0]);
 	  seq2.count = 0;
 	  getseq (fp2, &seq2);
@@ -399,7 +435,7 @@ join (fp1, fp2)
 	}
 
       /* Keep reading lines from file1 as long as they continue to
-	 match the current line from file2. */
+         match the current line from file2. */
       eof1 = 0;
       do
 	if (!getseq (fp1, &seq1))
@@ -411,7 +447,7 @@ join (fp1, fp2)
       while (!keycmp (&seq1.lines[seq1.count - 1], &seq2.lines[0]));
 
       /* Keep reading lines from file2 as long as they continue to
-	 match the current line from file1. */
+         match the current line from file1. */
       eof2 = 0;
       do
 	if (!getseq (fp2, &seq2))
@@ -452,22 +488,22 @@ join (fp1, fp2)
 
   if (print_unpairables_1 && seq1.count)
     {
-      prjoin(&seq1.lines[0], &blank2);
+      prjoin (&seq1.lines[0], &uni_blank);
       freeline (&seq1.lines[0]);
       while (get_line (fp1, &line))
 	{
-	  prjoin(&line, &blank2);
+	  prjoin (&line, &uni_blank);
 	  freeline (&line);
 	}
     }
 
   if (print_unpairables_2 && seq2.count)
     {
-      prjoin(&blank1, &seq2.lines[0]);
+      prjoin (&uni_blank, &seq2.lines[0]);
       freeline (&seq2.lines[0]);
       while (get_line (fp2, &line))
 	{
-	  prjoin(&blank1, &line);
+	  prjoin (&uni_blank, &line);
 	  freeline (&line);
 	}
     }
@@ -519,16 +555,7 @@ add_field_list (str)
       if (*str == ',' || ISBLANK (*str))
 	{
 	  added += add_field (file, field);
-	  switch (file)
-	    {
-	    case 1:
-	      blank1.nfields = max (blank1.nfields, field);
-	      break;
-
-	    case 2:
-	      blank2.nfields = max (blank2.nfields, field);
-	      break;
-	  }
+	  uni_blank.nfields = max (uni_blank.nfields, field);
 	  file = field = -1;
 	  dot_found = 0;
 	}
@@ -553,6 +580,7 @@ add_field_list (str)
 	return 0;
     }
 
+  uni_blank.nfields = max (uni_blank.nfields, field);
   added += add_field (file, field);
   return added;
 }
@@ -565,13 +593,15 @@ make_blank (blank, count)
      int count;
 {
   int i;
-  blank->beg = xmalloc(blank->nfields + 1);
-  blank->fields = (struct field *)xmalloc(sizeof(struct field) * count);
+  blank->nfields = count;
+  blank->beg = xmalloc (blank->nfields + 1);
+  blank->fields = (struct field *) xmalloc (sizeof (struct field) * count);
   for (i = 0; i < blank->nfields; i++)
     {
-    blank->beg[i] = '\t';
-    blank->fields[i].lim = blank->fields[i].beg = &blank->beg[i];
-  }
+      blank->beg[i] = '\t';
+      blank->fields[i].beg = &blank->beg[i];
+      blank->fields[i].len = 0;
+    }
   blank->beg[i] = '\0';
   blank->lim = &blank->beg[i];
 }
@@ -585,17 +615,13 @@ main (argc, argv)
   FILE *fp1, *fp2;
   int optc, prev_optc = 0, nfiles, val;
 
-  blank1.nfields = 1;
-  blank2.nfields = 1;
-  
   program_name = argv[0];
 
-  parse_long_options (argc, argv, "join", version_string, usage);
+  /* Initialize this before parsing options.  In parsing options,
+     it may be increased.  */
+  uni_blank.nfields = 1;
 
-  /* Now that we've seen the options, we can construct the blank line
-     structures.  */
-  make_blank(&blank1, blank1.nfields);
-  make_blank(&blank2, blank2.nfields);
+  parse_long_options (argc, argv, "join", version_string, usage);
 
   nfiles = 0;
   print_pairables = 1;
@@ -615,7 +641,7 @@ main (argc, argv)
 	  else if (val == 2)
 	    print_unpairables_2 = 1;
 	  else
-	    error (2, 0, "invalid file number for `-a'");
+	    error (2, 0, _("invalid file number for `-a'"));
 	  break;
 
 	case 'e':
@@ -625,27 +651,27 @@ main (argc, argv)
 	case '1':
 	  val = atoi (optarg);
 	  if (val <= 0)
-	    error (2, 0, "invalid field number for `-1'");
+	    error (2, 0, _("invalid field number for `-1'"));
 	  join_field_1 = val - 1;
 	  break;
 
 	case '2':
 	  val = atoi (optarg);
 	  if (val <= 0)
-	    error (2, 0, "invalid field number for `-2'");
+	    error (2, 0, _("invalid field number for `-2'"));
 	  join_field_2 = val - 1;
 	  break;
 
 	case 'j':
 	  val = atoi (optarg);
 	  if (val <= 0)
-	    error (2, 0, "invalid field number for `-j'");
+	    error (2, 0, _("invalid field number for `-j'"));
 	  join_field_1 = join_field_2 = val - 1;
 	  break;
 
 	case 'o':
 	  if (add_field_list (optarg) == 0)
-	    error (2, 0, "invalid field list for `-o'");
+	    error (2, 0, _("invalid field list for `-o'"));
 	  break;
 
 	case 't':
@@ -659,11 +685,11 @@ main (argc, argv)
 	  else if (val == 2)
 	    print_unpairables_2 = 1;
 	  else
-	    error (2, 0, "invalid file number for `-v'");
+	    error (2, 0, _("invalid file number for `-v'"));
 	  print_pairables = 0;
 	  break;
 
-	case 1:			/* Non-option argument. */
+	case 1:		/* Non-option argument. */
 	  if (prev_optc == 'o')
 	    {
 	      /* Might be continuation of args to -o. */
@@ -682,6 +708,10 @@ main (argc, argv)
       prev_optc = optc;
     }
 
+  /* Now that we've seen the options, we can construct the blank line
+     structure.  */
+  make_blank (&uni_blank, uni_blank.nfields);
+
   if (nfiles != 2)
     usage (1);
 
@@ -692,13 +722,13 @@ main (argc, argv)
   if (!fp2)
     error (1, errno, "%s", names[1]);
   if (fp1 == fp2)
-    error (1, errno, "both files cannot be standard input");
+    error (1, errno, _("both files cannot be standard input"));
   join (fp1, fp2);
 
   if ((fp1 == stdin || fp2 == stdin) && fclose (stdin) == EOF)
     error (1, errno, "-");
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
 
   exit (0);
 }
@@ -708,15 +738,18 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... FILE1 FILE2\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+For each pair of input lines with identical join fields, write a line to\n\
+standard output.  The default join field is the first, delimited\n\
+by whitespace.  When FILE1 or FILE2 (not both) is -, read standard input.\n\
 \n\
   -a SIDE          print unpairable lines coming from file SIDE\n\
   -e EMPTY         replace missing input fields with EMPTY\n\
@@ -728,14 +761,13 @@ Usage: %s [OPTION]... FILE1 FILE2\n\
   --help           display this help and exit\n\
   --version        output version information and exit\n\
 \n\
-When FILE1 or FILE2 is -, not both, read standard input.  SIDE is 1\n\
-for FILE1 or 2 for FILE2.  Unless -t CHAR is given, leading blanks\n\
+SIDE is 1 for FILE1 or 2 for FILE2.  Unless -t CHAR is given, leading blanks\n\
 separate fields and are ignored, else fields are separated by CHAR.\n\
 Any FIELD is a field number counted from 1.  FORMAT is one or more\n\
 comma or blank separated specifications, each being `SIDE.FIELD'.\n\
 Default FORMAT outputs the join field, the remaining fields from\n\
 FILE1, the remaining fields from FILE2, all separated by CHAR.\n\
-");
+"));
     }
   exit (status);
 }
