@@ -1,5 +1,5 @@
 /* od -- dump files in octal and other formats
-   Copyright (C) 1992 Free Software Foundation, Inc.
+   Copyright (C) 1992, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Jim Meyering.  */
 
@@ -40,6 +40,8 @@ char *alloca ();
 #include <sys/types.h>
 #include "system.h"
 #include "version.h"
+#include "xstrtoul.h"
+#include "error.h"
 
 #if defined(__GNUC__) || defined(STDC_HEADERS)
 #include <float.h>
@@ -69,6 +71,11 @@ typedef double LONG_DOUBLE;
 #ifndef ULONG_MAX
 #define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
 #endif
+#ifndef OFF_T_MAX
+/* FIXME: is there a way to do this without relying on the
+   `8 bits per byte' assumption?  */
+#define OFF_T_MAX (~((off_t)1 << (sizeof (off_t) * 8 - 1)))
+#endif
 
 #define STREQ(a,b) (strcmp((a), (b)) == 0)
 
@@ -100,7 +107,6 @@ typedef double LONG_DOUBLE;
 
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 
 enum size_spec
   {
@@ -109,9 +115,9 @@ enum size_spec
     SHORT,
     INT,
     LONG,
-    FP_SINGLE,
-    FP_DOUBLE,
-    FP_LONG_DOUBLE
+    FLOAT_SINGLE,
+    FLOAT_DOUBLE,
+    FLOAT_LONG_DOUBLE
   };
 
 enum output_format
@@ -125,12 +131,6 @@ enum output_format
     CHARACTER
   };
 
-enum strtoul_error
-  {
-    UINT_OK, UINT_INVALID, UINT_INVALID_SUFFIX_CHAR, UINT_OVERFLOW
-  };
-typedef enum strtoul_error strtoul_error;
-
 /* Each output format specification (from POSIX `-t spec' or from
    old-style options) is represented by one of these structures.  */
 struct tspec
@@ -220,7 +220,7 @@ static long int pseudo_offset;
 static const char *(*format_address) (/* long unsigned int */);
 
 /* The number of input bytes to skip before formatting and writing.  */
-static unsigned long int n_bytes_to_skip = 0;
+static off_t n_bytes_to_skip = 0;
 
 /* When non-zero, MAX_BYTES_TO_FORMAT is the maximum number of bytes
    to be read and formatted.  Otherwise all input is formatted.  */
@@ -228,7 +228,7 @@ static int limit_bytes_to_format = 0;
 
 /* The maximum number of bytes that will be formatted.  This
    value is used only when LIMIT_BYTES_TO_FORMAT is non-zero.  */
-static unsigned long int max_bytes_to_format;
+static off_t max_bytes_to_format;
 
 /* When non-zero and two or more consecutive blocks are equal, format
    only the first block and output an asterisk alone on the following
@@ -302,16 +302,18 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
   or:  %s --traditional [FILE] [[+]OFFSET [[+]LABEL]]\n\
-",
+"),
 	      program_name, program_name);
-      printf ("\
+      printf (_("\
+Write an unambiguous representation, octal bytes by default, of FILE\n\
+to standard output.  With no FILE, or when FILE is -, read standard input.\n\
 \n\
   -A, --address-radix=RADIX   decide how file offsets are printed\n\
   -N, --read-bytes=BYTES      limit dump to BYTES input bytes per file\n\
@@ -320,8 +322,8 @@ Usage: %s [OPTION]... [FILE]...\n\
   -t, --format=TYPE           select output format or formats\n\
   -v, --output-duplicates     do not use * to mark line suppression\n\
   -w, --width[=BYTES]         output BYTES bytes per output line\n\
-      --help                  display this help and exit\n\
       --traditional           accept arguments in pre-POSIX form\n\
+      --help                  display this help and exit\n\
       --version               output version information and exit\n\
 \n\
 Pre-POSIX format specifications may be intermixed, they accumulate:\n\
@@ -335,8 +337,8 @@ Pre-POSIX format specifications may be intermixed, they accumulate:\n\
   -l   same as -t d4, select decimal longs\n\
   -o   same as -t o2, select octal shorts\n\
   -x   same as -t x2, select hexadecimal shorts\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 For older syntax (second call format), OFFSET means -j OFFSET.  LABEL\n\
 is the pseudo-address at first byte printed, incremented when dump is\n\
@@ -362,9 +364,8 @@ RADIX is d for decimal, o for octal, x for hexadecimal or n for none.\n\
 BYTES is hexadecimal with 0x or 0X prefix, it is multiplied by 512\n\
 with b suffix, by 1024 with k and by 1048576 with m.  -s without a\n\
 number implies 3.  -w without a number implies 32.  By default, od\n\
-uses -A o -t d2 -w 16.  With no FILE, or when FILE is -, read standard\n\
-input.\n\
-");
+uses -A o -t d2 -w 16.\n\
+"));
     }
   exit (status);
 }
@@ -400,97 +401,6 @@ lcm (u, v)
   return u * v / t;
 }
 
-static strtoul_error
-my_strtoul (s, base, val, allow_bkm_suffix)
-     const char *s;
-     int base;
-     long unsigned int *val;
-     int allow_bkm_suffix;
-{
-  char *p;
-  unsigned long int tmp;
-
-  assert (0 <= base && base <= 36);
-
-  errno = 0;
-  tmp = strtoul (s, &p, base);
-  if (errno != 0)
-    return UINT_OVERFLOW;
-  if (p == s)
-    return UINT_INVALID;
-  if (!allow_bkm_suffix)
-    {
-      if (*p == '\0')
-	{
-	  *val = tmp;
-	  return UINT_OK;
-	}
-      else
-	return UINT_INVALID_SUFFIX_CHAR;
-    }
-
-  switch (*p)
-    {
-    case '\0':
-      break;
-
-#define BKM_SCALE(x,scale_factor,error_return)		\
-      do						\
-	{						\
-	  if (x > (double) ULONG_MAX / scale_factor)	\
-	    return error_return;			\
-	  x *= scale_factor;				\
-	}						\
-      while (0)
-
-    case 'b':
-      BKM_SCALE (tmp, 512, UINT_OVERFLOW);
-      break;
-
-    case 'k':
-      BKM_SCALE (tmp, 1024, UINT_OVERFLOW);
-      break;
-
-    case 'm':
-      BKM_SCALE (tmp, 1024 * 1024, UINT_OVERFLOW);
-      break;
-
-    default:
-      return UINT_INVALID_SUFFIX_CHAR;
-      break;
-    }
-
-  *val = tmp;
-  return UINT_OK;
-}
-
-static void
-uint_fatal_error (str, argument_type_string, err)
-     const char *str;
-     const char *argument_type_string;
-     strtoul_error err;
-{
-  switch (err)
-    {
-    case UINT_OK:
-      abort ();
-
-    case UINT_INVALID:
-      error (2, 0, "invalid %s `%s'", argument_type_string, str);
-      break;
-
-    case UINT_INVALID_SUFFIX_CHAR:
-      error (2, 0, "invalid character following %s `%s'",
-	     argument_type_string, str);
-      break;
-
-    case UINT_OVERFLOW:
-      error (2, 0, "%s `%s' larger than maximum unsigned long",
-	     argument_type_string, str);
-      break;
-    }
-}
-
 static void
 print_s_char (n_bytes, block, fmt_string)
      long unsigned int n_bytes;
@@ -877,7 +787,7 @@ decode_one_format (s, next, tspec)
 	case SHORT:
 	  print_function = (fmt == SIGNED_DECIMAL
 			    ? print_s_short
-			    : print_short);;
+			    : print_short);
 	  break;
 
 	case INT:
@@ -931,7 +841,7 @@ decode_one_format (s, next, tspec)
 
       switch (size_spec)
 	{
-	case FP_SINGLE:
+	case FLOAT_SINGLE:
 	  print_function = print_float;
 	  /* Don't use %#e; not all systems support it.  */
 	  pre_fmt_string = "%%%d.%de%%c";
@@ -940,7 +850,7 @@ decode_one_format (s, next, tspec)
 		   FLT_DIG + 8, FLT_DIG);
 	  break;
 
-	case FP_DOUBLE:
+	case FLOAT_DOUBLE:
 	  print_function = print_double;
 	  pre_fmt_string = "%%%d.%de%%c";
 	  fmt_string = xmalloc (strlen (pre_fmt_string));
@@ -949,7 +859,7 @@ decode_one_format (s, next, tspec)
 	  break;
 
 #ifdef HAVE_LONG_DOUBLE
-	case FP_LONG_DOUBLE:
+	case FLOAT_LONG_DOUBLE:
 	  print_function = print_long_double;
 	  pre_fmt_string = "%%%d.%dle%%c";
 	  fmt_string = xmalloc (strlen (pre_fmt_string));
@@ -1022,7 +932,7 @@ decode_format_string (s)
 						   * sizeof (struct tspec)));
 	}
 
-      bcopy ((char *) &tspec, (char *) &spec[n_specs], sizeof (struct tspec));
+      memcpy ((char *) &spec[n_specs], (char *) &tspec, sizeof (struct tspec));
       ++n_specs;
     }
 
@@ -1039,7 +949,7 @@ decode_format_string (s)
 
 static int
 skip (n_skip)
-     long unsigned int n_skip;
+     off_t n_skip;
 {
   int err;
 
@@ -1051,7 +961,7 @@ skip (n_skip)
 
       if (STREQ (*file_list, "-"))
 	{
-	  input_filename = "standard input";
+	  input_filename = _("standard input");
 	  in_stream = stdin;
 	  have_read_stdin = 1;
 	}
@@ -1134,7 +1044,7 @@ skip (n_skip)
     }
 
   if (n_skip != 0)
-    error (2, 0, "cannot skip past end of combined input");
+    error (2, 0, _("cannot skip past end of combined input"));
 
   return err;
 }
@@ -1191,7 +1101,7 @@ write_block (current_offset, n_bytes, prev_block, curr_block)
   static int first = 1;
   static int prev_pair_equal = 0;
 
-#define EQUAL_BLOCKS(b1, b2) (bcmp ((b1), (b2), bytes_per_block) == 0)
+#define EQUAL_BLOCKS(b1, b2) (memcmp ((b1), (b2), bytes_per_block) == 0)
 
   if (abbreviate_duplicate_blocks
       && !first && n_bytes == bytes_per_block
@@ -1210,7 +1120,7 @@ write_block (current_offset, n_bytes, prev_block, curr_block)
     }
   else
     {
-      int i;
+      unsigned int i;
 
       prev_pair_equal = 0;
       for (i = 0; i < n_specs; i++)
@@ -1251,7 +1161,7 @@ check_and_close ()
 
   if (ferror (stdout))
     {
-      error (0, errno, "standard output");
+      error (0, errno, _("standard output"));
       err = 1;
     }
 
@@ -1300,7 +1210,7 @@ read_char (c)
 
 	  if (STREQ (*file_list, "-"))
 	    {
-	      input_filename = "standard input";
+	      input_filename = _("standard input");
 	      in_stream = stdin;
 	      have_read_stdin = 1;
 	    }
@@ -1375,7 +1285,7 @@ read_block (n, block, n_bytes_in_buffer)
 
 	  if (STREQ (*file_list, "-"))
 	    {
-	      input_filename = "standard input";
+	      input_filename = _("standard input");
 	      in_stream = stdin;
 	      have_read_stdin = 1;
 	    }
@@ -1400,7 +1310,7 @@ read_block (n, block, n_bytes_in_buffer)
 static int
 get_lcm ()
 {
-  int i;
+  unsigned int i;
   int l_c_m = 1;
 
   for (i = 0; i < n_specs; i++)
@@ -1411,13 +1321,14 @@ get_lcm ()
 /* If S is a valid pre-POSIX offset specification with an optional leading '+'
    return the offset it denotes.  Otherwise, return -1.  */
 
-long int
+off_t
 parse_old_offset (s)
      const char *s;
 {
   int radix;
-  char *suffix;
-  long offset;
+  off_t offset;
+  enum strtol_error s_err;
+  long unsigned int tmp;
 
   if (*s == '\0')
     return -1;
@@ -1429,7 +1340,7 @@ parse_old_offset (s)
   /* Determine the radix we'll use to interpret S.  If there is a `.',
      it's decimal, otherwise, if the string begins with `0X'or `0x',
      it's hexadecimal, else octal.  */
-  if (index (s, '.') != NULL)
+  if (strchr (s, '.') != NULL)
     radix = 10;
   else
     {
@@ -1438,32 +1349,15 @@ parse_old_offset (s)
       else
 	radix = 8;
     }
-  offset = strtoul (s, &suffix, radix);
-  if (suffix == s || errno != 0)
-    return -1;
-  if (*suffix == '.')
-    ++suffix;
-  switch (*suffix)
+
+  s_err = xstrtoul (s, NULL, radix, &tmp, "Bb");
+  if (s_err != LONGINT_OK)
     {
-    case 'b':
-      BKM_SCALE (offset, 512, -1);
-      ++suffix;
-      break;
-
-    case 'B':
-      BKM_SCALE (offset, 1024, -1);
-      ++suffix;
-      break;
-
-    default:
-      /* empty */
-      break;
+      STRTOL_FAIL_WARN (s, _("old-style offset"), s_err);
+      return -1;
     }
-
-  if (*suffix != '\0')
-    return -1;
-  else
-    return offset;
+  offset = tmp;
+  return offset;
 }
 
 /* Read a chunk of size BYTES_PER_BLOCK from the input files, write the
@@ -1482,11 +1376,11 @@ static int
 dump ()
 {
   char *block[2];
-  unsigned long int current_offset;
+  off_t current_offset;
+  off_t end_offset;
   int idx;
   int err;
   size_t n_bytes_read;
-  size_t end_offset;
 
 #ifdef lint  /* Suppress `used before initialized' warning.  */
   end_offset = 0;
@@ -1503,11 +1397,16 @@ dump ()
     {
       end_offset = n_bytes_to_skip + max_bytes_to_format;
 
-      n_bytes_read = 0;
-      while (current_offset < end_offset)
+      while (1)
 	{
 	  size_t n_needed;
-	  n_needed = MIN (end_offset - current_offset, bytes_per_block);
+	  if (current_offset >= end_offset)
+	    {
+	      n_bytes_read = 0;
+	      break;
+	    }
+	  n_needed = MIN (end_offset - current_offset,
+			  (off_t) bytes_per_block);
 	  err |= read_block (n_needed, block[idx], &n_bytes_read);
 	  if (n_bytes_read < bytes_per_block)
 	    break;
@@ -1544,7 +1443,7 @@ dump ()
 	 is at least as large as n_bytes_read.  */
       bytes_to_write = l_c_m * (int) ((n_bytes_read + l_c_m - 1) / l_c_m);
 
-      bzero (block[idx] + n_bytes_read, bytes_to_write - n_bytes_read);
+      memset (block[idx] + n_bytes_read, 0, bytes_to_write - n_bytes_read);
       write_block (current_offset, bytes_to_write,
 		   block[!idx], block[idx]);
       current_offset += n_bytes_read;
@@ -1569,15 +1468,15 @@ dump ()
 static int
 dump_strings ()
 {
-  int bufsize = MAX (100, string_min);
+  size_t bufsize = MAX (100, string_min);
   char *buf = xmalloc (bufsize);
-  unsigned long address = n_bytes_to_skip;
+  off_t address = n_bytes_to_skip;
   int err;
 
   err = 0;
   while (1)
     {
-      int i;
+      unsigned int i;
       int c;
 
       /* See if the next `string_min' chars are all printing chars.  */
@@ -1688,7 +1587,7 @@ main (argc, argv)
 {
   int c;
   int n_files;
-  int i;
+  unsigned int i;
   unsigned int l_c_m;
   unsigned int address_pad_len;
   unsigned long int desired_width;
@@ -1717,12 +1616,12 @@ main (argc, argv)
   for (i = 0; i <= MAX_FP_TYPE_SIZE; i++)
     fp_type_size[i] = NO_SIZE;
 
-  fp_type_size[sizeof (float)] = FP_SINGLE;
+  fp_type_size[sizeof (float)] = FLOAT_SINGLE;
   /* The array entry for `double' is filled in after that for LONG_DOUBLE
      so that if `long double' is the same type or if long double isn't
-     supported FP_LONG_DOUBLE will never be used.  */
-  fp_type_size[sizeof (LONG_DOUBLE)] = FP_LONG_DOUBLE;
-  fp_type_size[sizeof (double)] = FP_DOUBLE;
+     supported FLOAT_LONG_DOUBLE will never be used.  */
+  fp_type_size[sizeof (LONG_DOUBLE)] = FLOAT_LONG_DOUBLE;
+  fp_type_size[sizeof (double)] = FLOAT_DOUBLE;
 
   n_specs = 0;
   n_specs_allocated = 5;
@@ -1737,7 +1636,8 @@ main (argc, argv)
 			   long_options, (int *) 0))
 	 != EOF)
     {
-      strtoul_error s_err;
+      unsigned long int tmp;
+      enum strtol_error s_err;
 
       switch (c)
 	{
@@ -1769,24 +1669,32 @@ main (argc, argv)
 	      break;
 	    default:
 	      error (2, 0,
-		     "invalid output address radix `%c'; it must be one character from [doxn]",
+		     _("invalid output address radix `%c'; it must be one character from [doxn]"),
 		     optarg[0]);
 	      break;
 	    }
 	  break;
 
 	case 'j':
-	  s_err = my_strtoul (optarg, 0, &n_bytes_to_skip, 1);
-	  if (s_err != UINT_OK)
-	    uint_fatal_error (optarg, "skip argument", s_err);
+	  s_err = xstrtoul (optarg, NULL, 0, &tmp, "bkm");
+	  n_bytes_to_skip = tmp;
+	  if (s_err != LONGINT_OK)
+	    STRTOL_FATAL_ERROR (optarg, _("skip argument"), s_err);
 	  break;
 
 	case 'N':
 	  limit_bytes_to_format = 1;
 
-	  s_err = my_strtoul (optarg, 0, &max_bytes_to_format, 1);
-	  if (s_err != UINT_OK)
-	    uint_fatal_error (optarg, "limit argument", s_err);
+	  /* FIXME: if off_t is long long and that's an 8-byte type,
+	     use xstrtouq here.  */
+	  s_err = xstrtoul (optarg, NULL, 0, &tmp, "bkm");
+	  max_bytes_to_format = tmp;
+	  if (s_err != LONGINT_OK)
+	    STRTOL_FATAL_ERROR (optarg, _("limit argument"), s_err);
+
+	  if (tmp > OFF_T_MAX)
+	    error (2, 0, _("specified number of bytes `%s' is larger than \
+the maximum\nrepresentable value of type off_t"), optarg);
 	  break;
 
 	case 's':
@@ -1794,16 +1702,16 @@ main (argc, argv)
 	    string_min = 3;
 	  else
 	    {
-	      s_err = my_strtoul (optarg, 0, &string_min, 1);
-	      if (s_err != UINT_OK)
-		uint_fatal_error (optarg, "minimum string length", s_err);
+	      s_err = xstrtoul (optarg, NULL, 0, &string_min, "bkm");
+	      if (s_err != LONGINT_OK)
+		STRTOL_FATAL_ERROR (optarg, _("minimum string length"), s_err);
 	    }
 	  ++flag_dump_strings;
 	  break;
 
 	case 't':
 	  if (decode_format_string (optarg))
-	    error (2, 0, "invalid type string `%s'", optarg);
+	    error (2, 0, _("invalid type string `%s'"), optarg);
 	  break;
 
 	case 'v':
@@ -1849,9 +1757,9 @@ main (argc, argv)
 	    }
 	  else
 	    {
-	      s_err = my_strtoul (optarg, 10, &desired_width, 0);
-	      if (s_err != UINT_OK)
-		error (2, 0, "invalid width specification `%s'", optarg);
+	      s_err = xstrtoul (optarg, NULL, 10, &desired_width, NULL);
+	      if (s_err != LONGINT_OK)
+		STRTOL_FATAL_ERROR (optarg, _("width specification"), s_err);
 	    }
 	  break;
 
@@ -1871,7 +1779,7 @@ main (argc, argv)
     usage (0);
 
   if (flag_dump_strings && n_specs > 0)
-    error (2, 0, "no type may be specified when dumping strings");
+    error (2, 0, _("no type may be specified when dumping strings"));
 
   n_files = argc - optind;
 
@@ -1883,7 +1791,7 @@ main (argc, argv)
 
   if (traditional)
     {
-      long int offset;
+      off_t offset;
 
       if (n_files == 1)
 	{
@@ -1896,7 +1804,7 @@ main (argc, argv)
 	}
       else if (n_files == 2)
 	{
-	  long int o1, o2;
+	  off_t o1, o2;
 	  if ((o1 = parse_old_offset (argv[optind])) >= 0
 	      && (o2 = parse_old_offset (argv[optind + 1])) >= 0)
 	    {
@@ -1916,14 +1824,14 @@ main (argc, argv)
 	  else
 	    {
 	      error (0, 0,
-		     "invalid second operand in compatibility mode `%s'",
+		     _("invalid second operand in compatibility mode `%s'"),
 		     argv[optind + 1]);
 	      usage (1);
 	    }
 	}
       else if (n_files == 3)
 	{
-	  long int o1, o2;
+	  off_t o1, o2;
 	  if ((o1 = parse_old_offset (argv[optind + 1])) >= 0
 	      && (o2 = parse_old_offset (argv[optind + 2])) >= 0)
 	    {
@@ -1937,14 +1845,14 @@ main (argc, argv)
 	  else
 	    {
 	      error (0, 0,
-	      "in compatibility mode the last 2 arguments must be offsets");
+	      _("in compatibility mode the last 2 arguments must be offsets"));
 	      usage (1);
 	    }
 	}
       else
 	{
 	  error (0, 0,
-	     "in compatibility mode there may be no more than 3 arguments");
+	     _("in compatibility mode there may be no more than 3 arguments"));
 	  usage (1);
 	}
 
@@ -2008,7 +1916,7 @@ main (argc, argv)
 	bytes_per_block = desired_width;
       else
 	{
-	  error (0, 0, "warning: invalid width %d; using %d instead",
+	  error (0, 0, _("warning: invalid width %lu; using %d instead"),
 		 desired_width, l_c_m);
 	  bytes_per_block = l_c_m;
 	}
@@ -2024,7 +1932,7 @@ main (argc, argv)
 #ifdef DEBUG
   for (i = 0; i < n_specs; i++)
     {
-      printf ("%d: fmt=\"%s\" width=%d\n",
+      printf (_("%d: fmt=\"%s\" width=%d\n"),
 	      i, spec[i].fmt_string, width_bytes[spec[i].size]);
     }
 #endif
@@ -2034,10 +1942,10 @@ main (argc, argv)
 cleanup:;
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (2, errno, "standard input");
+    error (2, errno, _("standard input"));
 
   if (fclose (stdout) == EOF)
-    error (2, errno, "write error");
+    error (2, errno, _("write error"));
 
   exit (err);
 }
