@@ -1,5 +1,5 @@
 /* pr -- convert text files for printing.
-   Copyright (C) 1988, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1988, 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /*  Author: Pete TerMaat.  */
 
@@ -102,10 +102,10 @@
 #include <time.h>
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
 char *xmalloc ();
 char *xrealloc ();
-void error ();
 
 static int char_to_clump ();
 static int read_line ();
@@ -489,7 +489,7 @@ main (argc, argv)
 	      ++s;
 	      if (!ISDIGIT (*s))
 		{
-		  error (0, 0, "`+' requires a numeric argument");
+		  error (0, 0, _("`+' requires a numeric argument"));
 		  usage (2);
 		}
 	      /* FIXME: use strtol */
@@ -592,8 +592,8 @@ main (argc, argv)
 	      column_separator = *s;
 	      if (*++s)
 		{
-		  fprintf (stderr, "\
-%s: extra characters in the argument to the `-s' option: `%s'\n",
+		  fprintf (stderr, _("\
+%s: extra characters in the argument to the `-s' option: `%s'\n"),
 			   program_name, s);
 		  usage (2);
 		}
@@ -625,11 +625,11 @@ main (argc, argv)
 
   if (parallel_files && explicit_columns)
     error (1, 0,
-  "Cannot specify number of columns when printing in parallel.");
+  _("Cannot specify number of columns when printing in parallel."));
 
   if (parallel_files && print_across_flag)
     error (1, 0,
-  "Cannot specify both printing across and printing in parallel.");
+  _("Cannot specify both printing across and printing in parallel."));
 
   for ( ; optind < argc; optind++)
     {
@@ -656,9 +656,9 @@ main (argc, argv)
   cleanup ();
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "standard input");
+    error (1, errno, _("standard input"));
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
   if (failed_opens > 0)
     exit(1);
   exit (0);
@@ -683,8 +683,8 @@ getoptarg (arg, switch_char, character, number)
 	*number = atoi (arg);
       else
 	{
-	  fprintf (stderr, "\
-%s: extra characters in the argument to the `-%c' option: `%s'\n",
+	  fprintf (stderr, _("\
+%s: extra characters in the argument to the `-%c' option: `%s'\n"),
 		   program_name, switch_char, arg);
 	  usage (2);
 	}
@@ -748,7 +748,7 @@ init_parameters (number_of_files)
 		      (columns - 1) * chars_per_gutter) / columns;
 
   if (chars_per_column < 1)
-    error (1, 0, "page width too narrow");
+    error (1, 0, _("page width too narrow"));
 
   if (numbered_lines)
     {
@@ -816,7 +816,7 @@ init_fps (number_of_files, av)
 	}
       else
 	{
-	  p->name = "standard input";
+	  p->name = _("standard input");
 	  p->fp = stdin;
 	  have_read_stdin = TRUE;
 	  p->status = OPEN;
@@ -929,7 +929,7 @@ open_file (name, p)
 {
   if (!strcmp (name, "-"))
     {
-      p->name = "standard input";
+      p->name = _("standard input");
       p->fp = stdin;
       have_read_stdin = 1;
     }
@@ -1107,7 +1107,7 @@ init_header (filename, desc)
     free (header);
   header = (char *) xmalloc (chars_per_header * sizeof (char));
 
-  sprintf (header, "%s %s  %s Page", &t[4], &t[20], middle);
+  sprintf (header, _("%s %s  %s Page"), &t[4], &t[20], middle);
 }
 
 /* Set things up for printing a page
@@ -1633,6 +1633,10 @@ read_line (p)
   register int c, chars;
   int last_input_position;
 
+#ifdef lint  /* Suppress `used before initialized' warning.  */
+  chars = 0;
+#endif
+
   c = getc (p->fp);
 
   last_input_position = input_position;
@@ -1871,15 +1875,16 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Paginate or columnate FILE(s) for printing.\n\
 \n\
   +PAGE             begin printing with page PAGE\n\
   -COLUMN           produce COLUMN-column output and print columns down\n\
@@ -1905,7 +1910,7 @@ Usage: %s [OPTION]... [FILE]...\n\
 \n\
 -t implied by -l N when N < 10.  Without -s, columns are separated by\n\
 spaces.  With no FILE, or when FILE is -, read standard input.\n\
-");
+"));
     }
   exit (status);
 }
