@@ -1,5 +1,5 @@
 /* sort - sort lines of text (with all kinds of options).
-   Copyright (C) 1988, 1991, 1992, 1993, 1994 Free Software Foundation
+   Copyright (C) 1988, 1991, 1992, 1993, 1994, 1995 Free Software Foundation
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
    Written December 1988 by Mike Haertel.
    The author may be reached (Email) at the address mike@gnu.ai.mit.edu,
@@ -30,7 +30,7 @@
 #include "system.h"
 #include "version.h"
 #include "long-options.h"
-#include "safe-stat.h"
+#include "error.h"
 
 #ifdef _POSIX_VERSION
 #include <limits.h>
@@ -45,13 +45,19 @@ char *realloc ();
 void free ();
 #endif
 
-void error ();
 static void usage ();
 
+/* Undefine, to avoid warning about redefinition on some systems.  */
+#undef min
 #define min(a, b) ((a) < (b) ? (a) : (b))
+
 #define UCHAR_LIM (UCHAR_MAX + 1)
 #define UCHAR(c) ((unsigned char) (c))
 
+#ifndef DEFAULT_TMPDIR
+#define DEFAULT_TMPDIR "/tmp"
+#endif
+
 /* The kind of blanks for '-b' to skip in various options. */
 enum blanktype { bl_start, bl_end, bl_both };
 
@@ -100,17 +106,17 @@ static struct month
 
 /* Initial buffer size for in core sorting.  Will not grow unless a
    line longer than this is seen. */
-static int sortalloc =  524288;
+static int sortalloc = 512 * 1024;
 
 /* Initial buffer size for in core merge buffers.  Bear in mind that
    up to NMERGE * mergealloc bytes may be allocated for merge buffers. */
-static int mergealloc =  16384;
+static int mergealloc =  16 * 1024;
 
 /* Guess of average line length. */
 static int linelength = 30;
 
 /* Maximum number of elements for the array(s) of struct line's, in bytes.  */
-#define LINEALLOC 262144
+#define LINEALLOC (256 * 1024)
 
 /* Prefix for temporary file names. */
 static char *temp_file_prefix;
@@ -208,7 +214,7 @@ xmalloc (n)
   p = malloc (n);
   if (p == 0)
     {
-      error (0, 0, "virtual memory exhausted");
+      error (0, 0, _("virtual memory exhausted"));
       cleanup ();
       exit (2);
     }
@@ -235,7 +241,7 @@ xrealloc (p, n)
   p = realloc (p, n);
   if (p == 0)
     {
-      error (0, 0, "virtual memory exhausted");
+      error (0, 0, _("virtual memory exhausted"));
       cleanup ();
       exit (2);
     }
@@ -263,19 +269,30 @@ static void
 xfclose (fp)
      FILE *fp;
 {
-  fflush (fp);
-  if (fp != stdin && fp != stdout)
+  if (fp == stdin)
     {
-      if (fclose (fp) != 0)
+      /* Allow reading stdin from tty more than once. */
+      if (feof (fp))
+	clearerr (fp);
+    }
+  else if (fp == stdout)
+    {
+      if (fflush (fp) != 0)
 	{
-	  error (0, errno, "error closing file");
+	  error (0, errno, _("flushing file"));
 	  cleanup ();
 	  exit (2);
 	}
     }
   else
-    /* Allow reading stdin from tty more than once. */
-    clearerr (fp);
+    {
+      if (fclose (fp) != 0)
+	{
+	  error (0, errno, _("error closing file"));
+	  cleanup ();
+	  exit (2);
+	}
+    }
 }
 
 static void
@@ -286,7 +303,7 @@ xfwrite (buf, size, nelem, fp)
 {
   if (fwrite (buf, size, nelem, fp) != nelem)
     {
-      error (0, errno, "write error");
+      error (0, errno, _("write error"));
       cleanup ();
       exit (2);
     }
@@ -297,16 +314,23 @@ xfwrite (buf, size, nelem, fp)
 static char *
 tempname ()
 {
-  static int seq;
+  static unsigned int seq;
   int len = strlen (temp_file_prefix);
-  char *name = xmalloc (len + 16);
-  struct tempnode *node =
-  (struct tempnode *) xmalloc (sizeof (struct tempnode));
+  char *name = xmalloc (len + 1 + sizeof ("sort") - 1 + 5 + 5 + 1);
+  struct tempnode *node;
+
+  node = (struct tempnode *) xmalloc (sizeof (struct tempnode));
+  sprintf (name,
+	   "%s%ssort%5.5d%5.5d",
+	   temp_file_prefix,
+	   (len && temp_file_prefix[len - 1] != '/') ? "/" : "",
+	   (unsigned int) getpid () & 0xffff, seq);
+
+  /* Make sure that SEQ's value fits in 5 digits.  */
+  ++seq;
+  if (seq >= 100000)
+    seq = 0;
 
-  if (len && temp_file_prefix[len - 1] != '/')
-    sprintf (name, "%s/sort%5.5d%5.5d", temp_file_prefix, getpid (), ++seq);
-  else
-    sprintf (name, "%ssort%5.5d%5.5d", temp_file_prefix, getpid (), ++seq);
   node->name = name;
   node->next = temphead.next;
   temphead.next = node;
@@ -383,7 +407,7 @@ fillbuf (buf, fp)
 {
   int cc;
 
-  bcopy (buf->buf + buf->used - buf->left, buf->buf, buf->left);
+  memmove (buf->buf, buf->buf + buf->used - buf->left, buf->left);
   buf->used = buf->left;
 
   while (!feof (fp) && (buf->used == 0 || !memchr (buf->buf, '\n', buf->used)))
@@ -396,7 +420,7 @@ fillbuf (buf, fp)
       cc = fread (buf->buf + buf->used, 1, buf->alloc - buf->used, fp);
       if (ferror (fp))
 	{
-	  error (0, errno, "read error");
+	  error (0, errno, _("read error"));
 	  cleanup ();
 	  exit (2);
 	}
@@ -639,7 +663,8 @@ numcompare (a, b)
 {
   register int tmpa, tmpb, loga, logb, tmp;
 
-  tmpa = UCHAR (*a), tmpb = UCHAR (*b);
+  tmpa = UCHAR (*a);
+  tmpb = UCHAR (*b);
 
   while (blanks[tmpa])
     tmpa = UCHAR (*++a);
@@ -852,6 +877,8 @@ keycompare (a, b)
 		diff = translate[UCHAR (*--texta)] - translate[UCHAR (*--textb)];
 		break;
 	      }
+	    else if (texta == lima && textb < limb) diff = -1;
+	    else if (texta < lima && textb == limb) diff = 1;
 	  }
       else if (ignore)
 	while (texta < lima && textb < limb)
@@ -865,6 +892,8 @@ keycompare (a, b)
 		diff = *--texta - *--textb;
 		break;
 	      }
+	    else if (texta == lima && textb < limb) diff = -1;
+	    else if (texta < lima && textb == limb) diff = 1;
 	  }
       else if (translate)
 	while (texta < lima && textb < limb)
@@ -931,7 +960,8 @@ compare (a, b)
 }
 
 /* Check that the lines read from the given FP come in order.  Return
-   1 if they do and 0 if there is a disorder. */
+   1 if they do and 0 if there is a disorder.
+   FIXME: return number of first out-of-order line if not sorted.  */
 
 static int
 checkfp (fp)
@@ -941,8 +971,7 @@ checkfp (fp)
   struct lines lines;		/* Lines scanned from the buffer. */
   struct line temp;		/* Copy of previous line. */
   int cc;			/* Character count. */
-  int cmp;			/* Result of calling compare. */
-  int alloc, i, success = 1;
+  int alloc, sorted = 1;
 
   initbuf (&buf, mergealloc);
   initlines (&lines, mergealloc / linelength + 1,
@@ -951,55 +980,62 @@ checkfp (fp)
   temp.text = xmalloc (alloc);
 
   cc = fillbuf (&buf, fp);
+  if (cc == 0)
+    goto finish;
+
   findlines (&buf, &lines);
 
-  if (cc)
-    do
-      {
-	/* Compare each line in the buffer with its successor. */
-	for (i = 0; i < lines.used - 1; ++i)
-	  {
-	    cmp = compare (&lines.lines[i], &lines.lines[i + 1]);
-	    if ((unique && cmp >= 0) || (cmp > 0))
-	      {
-		success = 0;
-		goto finish;
-	      }
-	  }
+  while (1)
+    {
+      struct line *prev_line;	/* Pointer to previous line. */
+      int cmp;			/* Result of calling compare. */
+      int i;
 
-	/* Save the last line of the buffer and refill the buffer. */
-	if (lines.lines[lines.used - 1].length > alloc)
-	  {
-	    while (lines.lines[lines.used - 1].length + 1 > alloc)
-	      alloc *= 2;
-	    temp.text = xrealloc (temp.text, alloc);
-	  }
-	bcopy (lines.lines[lines.used - 1].text, temp.text,
-	       lines.lines[lines.used - 1].length + 1);
-	temp.length = lines.lines[lines.used - 1].length;
+      /* Compare each line in the buffer with its successor. */
+      for (i = 0; i < lines.used - 1; ++i)
+	{
+	  cmp = compare (&lines.lines[i], &lines.lines[i + 1]);
+	  if ((unique && cmp >= 0) || (cmp > 0))
+	    {
+	      sorted = 0;
+	      goto finish;
+	    }
+	}
 
-	cc = fillbuf (&buf, fp);
-	if (cc)
-	  {
-	    findlines (&buf, &lines);
-	    /* Make sure the line saved from the old buffer contents is
-	       less than or equal to the first line of the new buffer. */
-	    cmp = compare (&temp, &lines.lines[0]);
-	    if ((unique && cmp >= 0) || (cmp > 0))
-	      {
-		success = 0;
-		break;
-	      }
-	  }
-      }
-    while (cc);
+      /* Save the last line of the buffer and refill the buffer. */
+      prev_line = lines.lines + (lines.used - 1);
+      if (prev_line->length > alloc)
+	{
+	  while (prev_line->length + 1 > alloc)
+	    alloc *= 2;
+	  temp.text = xrealloc (temp.text, alloc);
+	}
+      memcpy (temp.text, prev_line->text, prev_line->length + 1);
+      temp.length = prev_line->length;
+      temp.keybeg = temp.text + (prev_line->keybeg - prev_line->text);
+      temp.keylim = temp.text + (prev_line->keylim - prev_line->text);
+
+      cc = fillbuf (&buf, fp);
+      if (cc == 0)
+        break;
+
+      findlines (&buf, &lines);
+      /* Make sure the line saved from the old buffer contents is
+	 less than or equal to the first line of the new buffer. */
+      cmp = compare (&temp, &lines.lines[0]);
+      if ((unique && cmp >= 0) || (cmp > 0))
+	{
+	  sorted = 0;
+	  break;
+	}
+    }
 
 finish:
   xfclose (fp);
   free (buf.buf);
   free ((char *) lines.lines);
   free (temp.text);
-  return success;
+  return sorted;
 }
 
 /* Merge lines from FPS onto OFP.  NFPS cannot be greater than NMERGE.
@@ -1022,6 +1058,10 @@ mergefps (fps, nfps, ofp)
 				   output. */
   register int i, j, t;
 
+#ifdef lint  /* Suppress `used before initialized' warning.  */
+  savealloc = 0;
+#endif
+
   /* Allocate space for a saved line if necessary. */
   if (unique)
     {
@@ -1065,7 +1105,7 @@ mergefps (fps, nfps, ofp)
   /* Repeatedly output the smallest line until no input remains. */
   while (nfps)
     {
-      /* If uniqified output is turned out, output only the first of
+      /* If uniqified output is turned on, output only the first of
 	 an identical series of lines. */
       if (unique)
 	{
@@ -1084,7 +1124,7 @@ mergefps (fps, nfps, ofp)
 		  saved.text = xrealloc (saved.text, savealloc);
 		}
 	      saved.length = lines[ord[0]].lines[cur[ord[0]]].length;
-	      bcopy (lines[ord[0]].lines[cur[ord[0]]].text, saved.text,
+	      memcpy (saved.text, lines[ord[0]].lines[cur[ord[0]]].text,
 		     saved.length + 1);
 	      if (lines[ord[0]].lines[cur[ord[0]]].keybeg != NULL)
 		{
@@ -1221,7 +1261,7 @@ check (files, nfiles)
       fp = xfopen (files[i], "r");
       if (!checkfp (fp))
 	{
-	  printf ("%s: disorder on %s\n", program_name, files[i]);
+	  fprintf (stderr, _("%s: disorder on %s\n"), program_name, files[i]);
 	  ++disorders;
 	}
     }
@@ -1286,7 +1326,7 @@ sort (files, nfiles, ofp)
   int i, ntmp;
   FILE *fp, *tfp;
   struct tempnode *node;
-  int ntemp = 0;
+  int n_temp_files = 0;
   char **tempfiles;
 
   initbuf (&buf, sortalloc);
@@ -1309,11 +1349,11 @@ sort (files, nfiles, ofp)
 		xrealloc ((char *) tmp, ntmp * sizeof (struct line));
 	    }
 	  sortlines (lines.lines, lines.used, tmp);
-	  if (feof (fp) && !nfiles && !ntemp && !buf.left)
+	  if (feof (fp) && !nfiles && !n_temp_files && !buf.left)
 	    tfp = ofp;
 	  else
 	    {
-	      ++ntemp;
+	      ++n_temp_files;
 	      tfp = xfopen (tempname (), "w");
 	    }
 	  for (i = 0; i < lines.used; ++i)
@@ -1333,13 +1373,13 @@ sort (files, nfiles, ofp)
   free ((char *) lines.lines);
   free ((char *) tmp);
 
-  if (ntemp)
+  if (n_temp_files)
     {
-      tempfiles = (char **) xmalloc (ntemp * sizeof (char *));
-      i = ntemp;
+      tempfiles = (char **) xmalloc (n_temp_files * sizeof (char *));
+      i = n_temp_files;
       for (node = temphead.next; i > 0; node = node->next)
 	tempfiles[--i] = node->name;
-      merge (tempfiles, ntemp, ofp);
+      merge (tempfiles, n_temp_files, ofp);
       free ((char *) tempfiles);
     }
 }
@@ -1362,7 +1402,7 @@ static void
 badfieldspec (s)
      char *s;
 {
-  error (2, 0, "invalid field specification `%s'", s);
+  error (2, 0, _("invalid field specification `%s'"), s);
 }
 
 /* Handle interrupts and hangups. */
@@ -1425,6 +1465,10 @@ set_ordering (s, key, blanktype)
 	  break;
 	case 'n':
 	  key->numeric = 1;
+	  if (blanktype == bl_start || blanktype == bl_both)
+	    key->skipsblanks = 1;
+	  if (blanktype == bl_end || blanktype == bl_both)
+	    key->skipeblanks = 1;
 	  break;
 	case 'r':
 	  key->reverse = 1;
@@ -1461,7 +1505,7 @@ main (argc, argv)
 
   temp_file_prefix = getenv ("TMPDIR");
   if (temp_file_prefix == NULL)
-    temp_file_prefix = "/tmp";
+    temp_file_prefix = DEFAULT_TMPDIR;
 
 #ifdef _POSIX_VERSION
   newact.sa_handler = sighandler;
@@ -1512,7 +1556,7 @@ main (argc, argv)
 	  key->skipsblanks = key->skipeblanks = 0;
 	  key->numeric = key->month = key->reverse = 0;
 	  s = argv[i] + 1;
-	  if (!digits[UCHAR (*s)])
+	  if (! (digits[UCHAR (*s)] || (*s == '.' && digits[UCHAR (s[1])])))
 	    badfieldspec (argv[i]);
 	  for (t = 0; digits[UCHAR (*s)]; ++s)
 	    t = 10 * t + *s - '0';
@@ -1534,7 +1578,7 @@ main (argc, argv)
       else if (argv[i][0] == '-' && argv[i][1])
 	{
 	  s = argv[i] + 1;
-	  if (digits[UCHAR (*s)])
+	  if (digits[UCHAR (*s)] || (*s == '.' && digits[UCHAR (s[1])]))
 	    {
 	      if (!key)
 		usage (2);
@@ -1569,7 +1613,7 @@ main (argc, argv)
 		    else
 		      {
 			if (i == argc - 1)
-			  error (2, 0, "option `-k' requires an argument");
+			  error (2, 0, _("option `-k' requires an argument"));
 			else
 			  s = argv[++i];
 		      }
@@ -1612,13 +1656,22 @@ main (argc, argv)
 			/* Get POS2. */
 			for (t = 0; digits[UCHAR (*s)]; ++s)
 			  t = t * 10 + *s - '0';
+			if (t)
+			      t--;
 			t2 = 0;
+			/* FIXME: It's an error to specify `.'
+			   but no char-spec. */
 			if (*s == '.')
 			  {
 			    for (++s; digits[UCHAR (*s)]; ++s)
 			      t2 = t2 * 10 + *s - '0';
 			    if (t2)
-			      t--;
+			      t2--;
+			  }
+			else
+			  {
+			    /* `-k 2,3' is equivalent to `+1 -3'.  */
+			    ++t;
 			  }
 			key->eword = t;
 			key->echar = t2;
@@ -1638,7 +1691,7 @@ main (argc, argv)
 		    else
 		      {
 			if (i == argc - 1)
-			  error (2, 0, "option `-o' requires an argument");
+			  error (2, 0, _("option `-o' requires an argument"));
 			else
 			  outfile = argv[++i];
 		      }
@@ -1655,7 +1708,7 @@ main (argc, argv)
 			goto outer;
 		      }
 		    else
-		      error (2, 0, "option `-t' requires an argument");
+		      error (2, 0, _("option `-t' requires an argument"));
 		    break;
 		  case 'T':
 		    if (s[1])
@@ -1665,10 +1718,10 @@ main (argc, argv)
 			if (i < argc - 1)
 			  temp_file_prefix = argv[++i];
 			else
-			  error (2, 0, "option `-T' requires an argument");
+			  error (2, 0, _("option `-T' requires an argument"));
 		      }
 		    goto outer;
-		    break;
+		    /* break; */
 		  case 'u':
 		    unique = 1;
 		    break;
@@ -1677,7 +1730,7 @@ main (argc, argv)
 		       Solaris 2.  */
 		    goto outer;
 		  default:
-		    fprintf (stderr, "%s: unrecognized option `-%c'\n",
+		    fprintf (stderr, _("%s: unrecognized option `-%c'\n"),
 			     argv[0], *s);
 		    usage (2);
 		  }
@@ -1726,7 +1779,7 @@ main (argc, argv)
   if (strcmp (outfile, "-"))
     {
       struct stat outstat;
-      if (SAFE_STAT (outfile, &outstat) == 0)
+      if (stat (outfile, &outstat) == 0)
 	{
 	  /* The following code prevents a race condition when
 	     people use the brain dead shell programming idiom:
@@ -1747,7 +1800,7 @@ main (argc, argv)
 		{
 		  struct stat instat;
 		  if ((strcmp (files[i], "-")
-		       ? SAFE_STAT (files[i], &instat)
+		       ? stat (files[i], &instat)
 		       : fstat (fileno (stdin), &instat)) != 0)
 		    {
 		      error (0, errno, "%s", files[i]);
@@ -1796,12 +1849,12 @@ main (argc, argv)
      Solaris, Ultrix, and Irix.  This premature fflush makes the output
      reappear. --karl@cs.umb.edu  */
   if (fflush (ofp) < 0)
-    error (1, errno, "fflush", outfile);
+    error (1, errno, _("%s: write error"), outfile);
 
   if (have_read_stdin && fclose (stdin) == EOF)
-    error (1, errno, "-");
+    error (1, errno, outfile);
   if (ferror (stdout) || fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("%s: write error"), outfile);
 
   exit (0);
 }
@@ -1811,19 +1864,20 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Write sorted concatenation of all FILE(s) to standard output.\n\
 \n\
   +POS1 [-POS2]    start a key at POS1, end it before POS2\n\
   -M               compare (unknown) < `JAN' < ... < `DEC', imply -b\n\
-  -T DIRECT        use DIRECT for temporary files, not $TMPDIR or /tmp\n\
+  -T DIRECT        use DIRECT for temporary files, not $TMPDIR or %s\n\
   -b               ignore leading blanks in sort fields or keys\n\
   -c               check if given files already sorted, do not sort\n\
   -d               consider only [a-zA-Z0-9 ] characters in keys\n\
@@ -1846,7 +1900,8 @@ position in the field, both counted from zero.  OPTS is made up of one\n\
 or more of Mbdfinr, this effectively disable global -Mbdfinr settings\n\
 for that key.  If no key given, use the entire line as key.  With no\n\
 FILE, or when FILE is -, read standard input.\n\
-");
+")
+	      , DEFAULT_TMPDIR);
     }
   exit (status);
 }
