@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Include sys/types.h before this file.  */
 
@@ -67,39 +67,53 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+
 #ifndef _POSIX_VERSION
 off_t lseek ();
 #endif
 
-#if defined(HAVE_STRING_H) || defined(STDC_HEADERS)
-#if !defined(STDC_HEADERS) && defined(HAVE_MEMORY_H)
-#include <memory.h>
+#ifndef STDIN_FILENO
+#define STDIN_FILENO 0
 #endif
-#include <string.h>
-#ifndef index
-#define index strchr
+
+#ifndef STDOUT_FILENO
+#define STDOUT_FILENO 1
 #endif
-#ifndef rindex
-#define rindex strrchr
-#endif
-/* Don't define bcopy; we need one that can handle overlaps.  */
-#ifndef bzero
-#define bzero(s, n) memset ((s), 0, (n))
-#endif
-#ifndef bcmp
-#define bcmp(s1, s2, n) memcmp ((s1), (s2), (n))
+
+#ifndef STDERR_FILENO
+#define STDERR_FILENO 2
 #endif
+
+/* Don't use bcopy!  Use memmove if source and destination may overlap,
+   memcpy otherwise.  */
+
+#ifdef HAVE_STRING_H
+# if !STDC_HEADERS && HAVE_MEMORY_H
+#  include <memory.h>
+# endif
+# include <string.h>
 #else
-#include <strings.h>
+# include <strings.h>
 char *memchr ();
 #endif
 
 #include <errno.h>
+#ifndef errno
+extern int errno;
+#endif
+
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #else
 char *getenv ();
-extern int errno;
+#endif
+
+#ifndef EXIT_FAILURE
+# define EXIT_FAILURE 1
+#endif
+
+#ifndef EXIT_SUCCESS
+# define EXIT_SUCCESS 0
 #endif
 
 #ifdef HAVE_FCNTL_H
@@ -177,3 +191,7 @@ extern int errno;
 #define ISSPACE(c) (ISASCII (c) && isspace (c))
 #define ISUPPER(c) (ISASCII (c) && isupper (c))
 #define ISXDIGIT(c) (ISASCII (c) && isxdigit (c))
+
+/* Disable string localization for the time being.  */
+#undef _
+#define _(String) String
