@@ -1,5 +1,5 @@
 /* tac - concatenate and print files in reverse
-   Copyright (C) 1988, 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1988, 1989, 1990, 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Jay Lepreau (lepreau@cs.utah.edu).
    GNU enhancements by David MacKenzie (djm@gnu.ai.mit.edu). */
@@ -44,12 +44,17 @@ tac -r -s '.\|
 #include <regex.h>
 #include "system.h"
 #include "version.h"
+#include "error.h"
 
 #ifndef STDC_HEADERS
 char *malloc ();
 char *realloc ();
 #endif
 
+#ifndef DEFAULT_TMPDIR
+#define DEFAULT_TMPDIR "/tmp"
+#endif
+
 /* The number of bytes per atomic read. */
 #define INITIAL_READSIZE 8192
 
@@ -68,7 +73,6 @@ static void output ();
 static void save_stdin ();
 static void xwrite ();
 
-void error ();
 int full_write ();
 int safe_read ();
 
@@ -127,24 +131,24 @@ usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Write each FILE to standard output, last line first.\n\
+With no FILE, or when FILE is -, read standard input.\n\
 \n\
   -b, --before             attach the separator before instead of after\n\
   -r, --regex              interpret the separator as a regular expression\n\
   -s, --separator=STRING   use STRING as the separator instead of newline\n\
       --help               display this help and exit\n\
       --version            output version information and exit\n\
-\n\
-With no FILE, or when FILE is -, read standard input.\n\
-");
+"));
     }
   exit (status);
 }
@@ -180,7 +184,7 @@ main (argc, argv)
 	case 's':
 	  separator = optarg;
 	  if (*separator == 0)
-	    error (1, 0, "separator cannot be empty");
+	    error (1, 0, _("separator cannot be empty"));
 	  break;
 	default:
 	  usage (1);
@@ -248,7 +252,7 @@ main (argc, argv)
   if (have_read_stdin && close (0) < 0)
     error (1, errno, "-");
   if (close (1) < 0)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
   exit (errors);
 }
 
@@ -276,11 +280,11 @@ tac_stdin ()
      error, either. */
   if (fstat (0, &stats))
     {
-      error (0, errno, "standard input");
+      error (0, errno, _("standard input"));
       return 1;
     }
   if (S_ISREG (stats.st_mode))
-    return tac (0, "standard input");
+    return tac (0, _("standard input"));
 
 #ifdef _POSIX_VERSION
   newact.sa_handler = cleanup;
@@ -363,7 +367,7 @@ save_stdin ()
     {
       tempdir = getenv ("TMPDIR");
       if (tempdir == NULL)
-	tempdir = "/tmp";
+	tempdir = DEFAULT_TMPDIR;
       template = xmalloc (strlen (tempdir) + 11);
     }
   sprintf (template, "%s/tacXXXXXX", tempdir);
@@ -388,7 +392,7 @@ save_stdin ()
     }
   if (bytes_read == -1)
     {
-      error (0, errno, "read error");
+      error (0, errno, _("read error"));
       cleanup ();
     }
 }
@@ -488,7 +492,7 @@ tac (fd, file)
 	    match_start = buffer - 1;
 	  else if (ret == -2)
 	    {
-	      error (0, 0, "error in regular expression search");
+	      error (0, 0, _("error in regular expression search"));
 	      cleanup ();
 	    }
 	  else
@@ -546,8 +550,9 @@ tac (fd, file)
 	    }
 	  lseek (fd, file_pos, SEEK_SET);
 
-	  /* Shift the pending record data right to make room for the new. */
-	  bcopy (buffer, buffer + read_size, saved_record_size);
+	  /* Shift the pending record data right to make room for the new.
+	     The source and destination regions probably overlap.  */
+	  memmove (buffer + read_size, buffer, saved_record_size);
 	  past_end = buffer + read_size + saved_record_size;
 	  /* For non-regexp searches, avoid unneccessary scanning. */
 	  if (sentinel_length)
@@ -600,7 +605,7 @@ output (start, past_end)
 
   if (start == 0)
     {
-      xwrite (1, buffer, bytes_in_buffer);
+      xwrite (STDOUT_FILENO, buffer, bytes_in_buffer);
       bytes_in_buffer = 0;
       return;
     }
@@ -608,15 +613,15 @@ output (start, past_end)
   /* Write out as many full buffers as possible. */
   while (bytes_to_add >= bytes_available)
     {
-      bcopy (start, buffer + bytes_in_buffer, bytes_available);
+      memcpy (buffer + bytes_in_buffer, start, bytes_available);
       bytes_to_add -= bytes_available;
       start += bytes_available;
-      xwrite (1, buffer, WRITESIZE);
+      xwrite (STDOUT_FILENO, buffer, WRITESIZE);
       bytes_in_buffer = 0;
       bytes_available = WRITESIZE;
     }
 
-  bcopy (start, buffer + bytes_in_buffer, bytes_to_add);
+  memcpy (buffer + bytes_in_buffer, start, bytes_to_add);
   bytes_in_buffer += bytes_to_add;
 }
 
@@ -635,7 +640,7 @@ xwrite (desc, buffer, size)
 {
   if (full_write (desc, buffer, size) < 0)
     {
-      error (0, errno, "write error");
+      error (0, errno, _("write error"));
       cleanup ();
     }
 }
@@ -651,7 +656,7 @@ xmalloc (n)
   p = malloc (n);
   if (p == 0)
     {
-      error (0, 0, "virtual memory exhausted");
+      error (0, 0, _("virtual memory exhausted"));
       cleanup ();
     }
   return p;
@@ -667,7 +672,7 @@ xrealloc (p, n)
   p = realloc (p, n);
   if (p == 0)
     {
-      error (0, 0, "virtual memory exhausted");
+      error (0, 0, _("virtual memory exhausted"));
       cleanup ();
     }
   return p;
