@@ -1,5 +1,5 @@
 /* tail -- output the last part of file(s)
-   Copyright (C) 1989, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1989, 1990, 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Can display any amount of data, unlike the Unix version, which uses
    a fixed size buffer and therefore can only deliver a limited number
@@ -26,9 +26,7 @@
    -f, --follow		Loop forever trying to read more characters at the
 			end of the file, on the assumption that the file
 			is growing.  Ignored if reading from a pipe.
-   -k			Tail by N kilobytes.
-   -N, -l, -n, --lines=N	Tail by N lines.
-   -m			Tail by N megabytes.
+   -n, --lines=N	Tail by N lines.
    -q, --quiet, --silent	Never print filename headers.
    -v, --verbose		Always print filename headers.
 
@@ -51,8 +49,11 @@
 #include <assert.h>
 #include <getopt.h>
 #include <sys/types.h>
+
 #include "system.h"
 #include "version.h"
+#include "xstrtol.h"
+#include "error.h"
 
 /* Disable assertions.  Some systems have broken assert macros.  */
 #define NDEBUG 1
@@ -63,23 +64,23 @@
       assert ((fd) == 1);						\
       assert ((n_bytes) >= 0);						\
       if (n_bytes > 0 && fwrite ((buffer), 1, (n_bytes), stdout) == 0)	\
-	error (1, errno, "write error");				\
+	error (1, errno, _("write error"));				\
     }									\
   while (0)
 
-/* Number of items to tail. */
-#define DEFAULT_NUMBER 10
+/* Number of items to tail.  */
+#define DEFAULT_N_LINES 10
 
-/* Size of atomic reads. */
-#ifndef BUFSIZE
-#define BUFSIZE (512 * 8)
+/* Size of atomic reads.  */
+#ifndef BUFSIZ
+#define BUFSIZ (512 * 8)
 #endif
 
-/* Number of bytes per item we are printing.
-   If 0, tail in lines. */
-static int unit_size;
+/* If nonzero, interpret the numeric argument as the number of lines.
+   Otherwise, interpret it as the number of bytes.  */
+static int count_lines;
 
-/* If nonzero, read from the end of one file until killed. */
+/* If nonzero, read from the end of one file until killed.  */
 static int forever;
 
 /* If nonzero, read from the end of multiple files until killed.  */
@@ -91,20 +92,19 @@ static int *file_descs;
 /* Array of file sizes if forever_multiple is 1.  */
 static off_t *file_sizes;
 
-/* If nonzero, count from start of file instead of end. */
+/* If nonzero, count from start of file instead of end.  */
 static int from_start;
 
-/* If nonzero, print filename headers. */
+/* If nonzero, print filename headers.  */
 static int print_headers;
 
-/* When to print the filename banners. */
+/* When to print the filename banners.  */
 enum header_mode
 {
   multiple_files, always, never
 };
 
 char *xmalloc ();
-void error ();
 int safe_read ();
 
 static int file_lines ();
@@ -116,17 +116,15 @@ static int tail ();
 static int tail_bytes ();
 static int tail_file ();
 static int tail_lines ();
-static long atou();
 static long dump_remainder ();
 static void tail_forever ();
-static void parse_unit ();
 static void usage ();
 static void write_header ();
 
-/* The name this program was run with. */
+/* The name this program was run with.  */
 char *program_name;
 
-/* Nonzero if we have ever read standard input. */
+/* Nonzero if we have ever read standard input.  */
 static int have_read_stdin;
 
 /* If non-zero, display usage information and exit.  */
@@ -157,14 +155,15 @@ main (argc, argv)
   int exit_status = 0;
   /* If from_start, the number of items to skip before printing; otherwise,
      the number of items at the end of the file to print.  Initially, -1
-     means the value has not been set. */
-  long number = -1;
-  int c;			/* Option character. */
+     means the value has not been set.  */
+  off_t n_units = -1;
+  long int tmp_long;
+  int c;			/* Option character.  */
   int fileind;			/* Index in ARGV of first file name.  */
 
   program_name = argv[0];
   have_read_stdin = 0;
-  unit_size = 0;
+  count_lines = 1;
   forever = forever_multiple = from_start = print_headers = 0;
 
   if (argc > 1
@@ -172,40 +171,36 @@ main (argc, argv)
 	  || (argv[1][0] == '+' && (ISDIGIT (argv[1][1]) || argv[1][1] == 0))))
     {
       /* Old option syntax: a dash or plus, one or more digits (zero digits
-	 are acceptable with a plus), and one or more option letters. */
+	 are acceptable with a plus), and one or more option letters.  */
       if (argv[1][0] == '+')
 	from_start = 1;
-      if (argv[1][1] != 0)
+      if (argv[1][1] != '\0')
 	{
-	  for (number = 0, ++argv[1]; ISDIGIT (*argv[1]); ++argv[1])
-	    number = number * 10 + *argv[1] - '0';
-	  /* Parse any appended option letters. */
-	  while (*argv[1])
-	    {
-	      switch (*argv[1])
-		{
-		case 'b':
-		  unit_size = 512;
-		  break;
+	  strtol_error s_err;
+	  char *p;
 
+	  s_err = xstrtol (++argv[1], &p, 0, &tmp_long, "bkm");
+	  n_units = tmp_long;
+	  if (s_err == LONGINT_OVERFLOW)
+	    {
+	      STRTOL_FATAL_ERROR (argv[1], _("argument"), s_err);
+	    }
+	  /* Parse any appended option letters.  */
+	  while (*p)
+	    {
+	      switch (*p)
+		{
 		case 'c':
-		  unit_size = 1;
+		  /* Interpret N_UNITS as # of bytes.  */
+		  count_lines = 0;
 		  break;
 
 		case 'f':
 		  forever = 1;
 		  break;
 
-		case 'k':
-		  unit_size = 1024;
-		  break;
-
 		case 'l':
-		  unit_size = 0;
-		  break;
-
-		case 'm':
-		  unit_size = 1048576;
+		  count_lines = 1;
 		  break;
 
 		case 'q':
@@ -217,13 +212,13 @@ main (argc, argv)
 		  break;
 
 		default:
-		  error (0, 0, "unrecognized option `-%c'", *argv[1]);
+		  error (0, 0, _("unrecognized option `-%c'"), *p);
 		  usage (1);
 		}
-	      ++argv[1];
+	      ++p;
 	    }
 	}
-      /* Make the options we just parsed invisible to getopt. */
+      /* Make the options we just parsed invisible to getopt.  */
       argv[1] = argv[0];
       argv++;
       argc--;
@@ -232,28 +227,35 @@ main (argc, argv)
   while ((c = getopt_long (argc, argv, "c:n:fqv", long_options, (int *) 0))
 	 != EOF)
     {
+      strtol_error s_err;
+
       switch (c)
 	{
 	case 0:
 	  break;
 
 	case 'c':
-	  unit_size = 1;
-	  parse_unit (optarg);
+	  count_lines = 0;
 	  goto getnum;
+
 	case 'n':
-	  unit_size = 0;
+	  count_lines = 1;
 	getnum:
 	  if (*optarg == '+')
 	    {
 	      from_start = 1;
-	      ++optarg;
 	    }
-	  else if (*optarg == '-')
-	    ++optarg;
-	  number = atou (optarg);
-	  if (number == -1)
-	    error (1, 0, "invalid number `%s'", optarg);
+
+	  s_err = xstrtol (optarg, NULL, 0, &tmp_long, "bkm");
+	  if (tmp_long < 0)
+	    tmp_long = -tmp_long;
+	  n_units = tmp_long;
+	  if (s_err != LONGINT_OK)
+	    {
+	      STRTOL_FATAL_ERROR (optarg, (c == 'n'
+					   ? _("number of lines")
+					   : _("number of bytes")), s_err);
+	    }
 	  break;
 
 	case 'f':
@@ -282,21 +284,18 @@ main (argc, argv)
   if (show_help)
     usage (0);
 
-  if (number == -1)
-    number = DEFAULT_NUMBER;
+  if (n_units == -1)
+    n_units = DEFAULT_N_LINES;
 
-  /* To start printing with item `number' from the start of the file, skip
-     `number' - 1 items.  `tail +0' is actually meaningless, but for Unix
-     compatibility it's treated the same as `tail +1'. */
+  /* To start printing with item N_UNITS from the start of the file, skip
+     N_UNITS - 1 items.  `tail +0' is actually meaningless, but for Unix
+     compatibility it's treated the same as `tail +1'.  */
   if (from_start)
     {
-      if (number)
-	--number;
+      if (n_units)
+	--n_units;
     }
 
-  if (unit_size > 1)
-    number *= unit_size;
-
   fileind = optind;
 
   if (optind < argc - 1 && forever)
@@ -312,10 +311,10 @@ main (argc, argv)
     print_headers = 1;
 
   if (optind == argc)
-    exit_status |= tail_file ("-", number, 0);
+    exit_status |= tail_file ("-", n_units, 0);
 
   for (; optind < argc; ++optind)
-    exit_status |= tail_file (argv[optind], number, optind - fileind);
+    exit_status |= tail_file (argv[optind], n_units, optind - fileind);
 
   if (forever_multiple)
     tail_forever (argv + fileind, argc - fileind);
@@ -323,19 +322,19 @@ main (argc, argv)
   if (have_read_stdin && close (0) < 0)
     error (1, errno, "-");
   if (fclose (stdout) == EOF)
-    error (1, errno, "write error");
+    error (1, errno, _("write error"));
   exit (exit_status);
 }
 
-/* Display the last NUMBER units of file FILENAME.
+/* Display the last N_UNITS units of file FILENAME.
    "-" for FILENAME means the standard input.
    FILENUM is this file's index in the list of files the user gave.
-   Return 0 if successful, 1 if an error occurred. */
+   Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail_file (filename, number, filenum)
-     char *filename;
-     long number;
+tail_file (filename, n_units, filenum)
+     const char *filename;
+     off_t n_units;
      int filenum;
 {
   int fd, errors;
@@ -344,21 +343,21 @@ tail_file (filename, number, filenum)
   if (!strcmp (filename, "-"))
     {
       have_read_stdin = 1;
-      filename = "standard input";
+      filename = _("standard input");
       if (print_headers)
 	write_header (filename, NULL);
-      errors = tail (filename, 0, number);
+      errors = tail (filename, 0, n_units);
       if (forever_multiple)
 	{
 	  if (fstat (0, &stats) < 0)
 	    {
-	      error (0, errno, "standard input");
+	      error (0, errno, _("standard input"));
 	      errors = 1;
 	    }
 	  else if (!S_ISREG (stats.st_mode))
 	    {
 	      error (0, 0,
-		     "standard input: cannot follow end of non-regular file");
+		     _("standard input: cannot follow end of non-regular file"));
 	      errors = 1;
 	    }
 	  if (errors)
@@ -385,7 +384,7 @@ tail_file (filename, number, filenum)
 	{
 	  if (print_headers)
 	    write_header (filename, NULL);
-	  errors = tail (filename, fd, number);
+	  errors = tail (filename, fd, n_units);
 	  if (forever_multiple)
 	    {
 	      if (fstat (fd, &stats) < 0)
@@ -395,7 +394,8 @@ tail_file (filename, number, filenum)
 		}
 	      else if (!S_ISREG (stats.st_mode))
 		{
-		  error (0, 0, "%s: cannot follow end of non-regular file");
+		  error (0, 0, _("%s: cannot follow end of non-regular file"),
+			 filename);
 		  errors = 1;
 		}
 	      if (errors)
@@ -436,37 +436,37 @@ write_header (filename, comment)
   first_file = 0;
 }
 
-/* Display the last NUMBER units of file FILENAME, open for reading
+/* Display the last N_UNITS units of file FILENAME, open for reading
    in FD.
-   Return 0 if successful, 1 if an error occurred. */
+   Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail (filename, fd, number)
-     char *filename;
+tail (filename, fd, n_units)
+     const char *filename;
      int fd;
-     long number;
+     off_t n_units;
 {
-  if (unit_size)
-    return tail_bytes (filename, fd, number);
+  if (count_lines)
+    return tail_lines (filename, fd, (long) n_units);
   else
-    return tail_lines (filename, fd, number);
+    return tail_bytes (filename, fd, n_units);
 }
 
-/* Display the last part of file FILENAME, open for reading in FD,
-   using NUMBER characters.
-   Return 0 if successful, 1 if an error occurred. */
+/* Output the last N_BYTES bytes of file FILENAME open for reading in FD.
+   Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail_bytes (filename, fd, number)
-     char *filename;
+tail_bytes (filename, fd, n_bytes)
+     const char *filename;
      int fd;
-     long number;
+     off_t n_bytes;
 {
   struct stat stats;
 
+  /* FIXME: resolve this like in dd.c.  */
   /* Use fstat instead of checking for errno == ESPIPE because
      lseek doesn't work on some special files but doesn't return an
-     error, either. */
+     error, either.  */
   if (fstat (fd, &stats))
     {
       error (0, errno, "%s", filename);
@@ -476,8 +476,8 @@ tail_bytes (filename, fd, number)
   if (from_start)
     {
       if (S_ISREG (stats.st_mode))
-	lseek (fd, number, SEEK_SET);
-      else if (start_bytes (filename, fd, number))
+	lseek (fd, n_bytes, SEEK_CUR);
+      else if (start_bytes (filename, fd, n_bytes))
 	return 1;
       dump_remainder (filename, fd);
     }
@@ -485,33 +485,56 @@ tail_bytes (filename, fd, number)
     {
       if (S_ISREG (stats.st_mode))
 	{
-	  if (lseek (fd, 0L, SEEK_END) <= number)
-	    /* The file is shorter than we want, or just the right size, so
-	       print the whole file. */
-	    lseek (fd, 0L, SEEK_SET);
+	  off_t current_pos, end_pos;
+	  size_t bytes_remaining;
+
+	  if ((current_pos = lseek (fd, (off_t) 0, SEEK_CUR)) != -1
+	      && (end_pos = lseek (fd, (off_t) 0, SEEK_END)) != -1)
+	    {
+	      off_t diff;
+	      /* Be careful here.  The current position may actually be
+		 beyond the end of the file.  */
+	      bytes_remaining = (diff = end_pos - current_pos) < 0 ? 0 : diff;
+	    }
 	  else
-	    /* The file is longer than we want, so go back. */
-	    lseek (fd, -number, SEEK_END);
+	    {
+	      error (0, errno, "%s", filename);
+	      return 1;
+	    }
+
+	  if (bytes_remaining <= n_bytes)
+	    {
+	      /* From the current position to end of file, there are no
+		 more bytes than have been requested.  So reposition the
+		 file pointer to the incoming current position and print
+		 everything after that.  */
+	      lseek (fd, current_pos, SEEK_SET);
+	    }
+	  else
+	    {
+	      /* There are more bytes remaining than were requested.
+		 Back up.  */
+	      lseek (fd, -n_bytes, SEEK_END);
+	    }
 	  dump_remainder (filename, fd);
 	}
       else
-	return pipe_bytes (filename, fd, number);
+	return pipe_bytes (filename, fd, n_bytes);
     }
   return 0;
 }
 
-/* Display the last part of file FILENAME, open for reading on FD,
-   using NUMBER lines.
-   Return 0 if successful, 1 if an error occurred. */
+/* Output the last N_LINES lines of file FILENAME open for reading in FD.
+   Return 0 if successful, 1 if an error occurred.  */
 
 static int
-tail_lines (filename, fd, number)
-     char *filename;
+tail_lines (filename, fd, n_lines)
+     const char *filename;
      int fd;
-     long number;
+     long n_lines;
 {
   struct stat stats;
-  long length;
+  off_t length;
 
   if (fstat (fd, &stats))
     {
@@ -521,54 +544,61 @@ tail_lines (filename, fd, number)
 
   if (from_start)
     {
-      if (start_lines (filename, fd, number))
+      if (start_lines (filename, fd, n_lines))
 	return 1;
       dump_remainder (filename, fd);
     }
   else
     {
-      if (S_ISREG (stats.st_mode))
+      /* Use file_lines only if FD refers to a regular file with
+         its file pointer positioned at beginning of file.  */
+      /* FIXME: adding the lseek conjunct is a kludge.
+	 Once there's a reasonable test suite, fix the true culprit:
+	 file_lines.  file_lines shouldn't presume that the input
+	 file pointer is initially positioned to beginning of file.  */
+      if (S_ISREG (stats.st_mode)
+	  && lseek (fd, (off_t) 0, SEEK_CUR) == (off_t) 0)
 	{
-	  length = lseek (fd, 0L, SEEK_END);
-	  if (length != 0 && file_lines (filename, fd, number, length))
+	  length = lseek (fd, (off_t) 0, SEEK_END);
+	  if (length != 0 && file_lines (filename, fd, n_lines, length))
 	    return 1;
 	  dump_remainder (filename, fd);
 	}
       else
-	return pipe_lines (filename, fd, number);
+	return pipe_lines (filename, fd, n_lines);
     }
   return 0;
 }
 
-/* Print the last NUMBER lines from the end of file FD.
-   Go backward through the file, reading `BUFSIZE' bytes at a time (except
+/* Print the last N_LINES lines from the end of file FD.
+   Go backward through the file, reading `BUFSIZ' bytes at a time (except
    probably the first), until we hit the start of the file or have
    read NUMBER newlines.
    POS starts out as the length of the file (the offset of the last
    byte of the file + 1).
-   Return 0 if successful, 1 if an error occurred. */
+   Return 0 if successful, 1 if an error occurred.  */
 
 static int
-file_lines (filename, fd, number, pos)
-     char *filename;
+file_lines (filename, fd, n_lines, pos)
+     const char *filename;
      int fd;
-     long number;
-     long pos;
+     long n_lines;
+     off_t pos;
 {
-  char buffer[BUFSIZE];
+  char buffer[BUFSIZ];
   int bytes_read;
-  int i;			/* Index into `buffer' for scanning. */
+  int i;			/* Index into `buffer' for scanning.  */
 
-  if (number == 0)
+  if (n_lines == 0)
     return 0;
 
   /* Set `bytes_read' to the size of the last, probably partial, buffer;
-     0 < `bytes_read' <= `BUFSIZE'. */
-  bytes_read = pos % BUFSIZE;
+     0 < `bytes_read' <= `BUFSIZ'.  */
+  bytes_read = pos % BUFSIZ;
   if (bytes_read == 0)
-    bytes_read = BUFSIZE;
-  /* Make `pos' a multiple of `BUFSIZE' (0 if the file is short), so that all
-     reads will be on block boundaries, which might increase efficiency. */
+    bytes_read = BUFSIZ;
+  /* Make `pos' a multiple of `BUFSIZ' (0 if the file is short), so that all
+     reads will be on block boundaries, which might increase efficiency.  */
   pos -= bytes_read;
   lseek (fd, pos, SEEK_SET);
   bytes_read = safe_read (fd, buffer, bytes_read);
@@ -578,36 +608,36 @@ file_lines (filename, fd, number, pos)
       return 1;
     }
 
-  /* Count the incomplete line on files that don't end with a newline. */
+  /* Count the incomplete line on files that don't end with a newline.  */
   if (bytes_read && buffer[bytes_read - 1] != '\n')
-    --number;
+    --n_lines;
 
   do
     {
-      /* Scan backward, counting the newlines in this bufferfull. */
+      /* Scan backward, counting the newlines in this bufferfull.  */
       for (i = bytes_read - 1; i >= 0; i--)
 	{
-	  /* Have we counted the requested number of newlines yet? */
-	  if (buffer[i] == '\n' && number-- == 0)
+	  /* Have we counted the requested number of newlines yet?  */
+	  if (buffer[i] == '\n' && n_lines-- == 0)
 	    {
 	      /* If this newline wasn't the last character in the buffer,
-	         print the text after it. */
+	         print the text after it.  */
 	      if (i != bytes_read - 1)
-		XWRITE (1, &buffer[i + 1], bytes_read - (i + 1));
+		XWRITE (STDOUT_FILENO, &buffer[i + 1], bytes_read - (i + 1));
 	      return 0;
 	    }
 	}
-      /* Not enough newlines in that bufferfull. */
+      /* Not enough newlines in that bufferfull.  */
       if (pos == 0)
 	{
-	  /* Not enough lines in the file; print the entire file. */
-	  lseek (fd, 0L, SEEK_SET);
+	  /* Not enough lines in the file; print the entire file.  */
+	  lseek (fd, (off_t) 0, SEEK_SET);
 	  return 0;
 	}
-      pos -= BUFSIZE;
+      pos -= BUFSIZ;
       lseek (fd, pos, SEEK_SET);
     }
-  while ((bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0);
+  while ((bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0);
   if (bytes_read == -1)
     {
       error (0, errno, "%s", filename);
@@ -616,27 +646,27 @@ file_lines (filename, fd, number, pos)
   return 0;
 }
 
-/* Print the last NUMBER lines from the end of the standard input,
+/* Print the last N_LINES lines from the end of the standard input,
    open for reading as pipe FD.
    Buffer the text as a linked list of LBUFFERs, adding them as needed.
-   Return 0 if successful, 1 if an error occured. */
+   Return 0 if successful, 1 if an error occured.  */
 
 static int
-pipe_lines (filename, fd, number)
-     char *filename;
+pipe_lines (filename, fd, n_lines)
+     const char *filename;
      int fd;
-     long number;
+     long n_lines;
 {
   struct linebuffer
   {
     int nbytes, nlines;
-    char buffer[BUFSIZE];
+    char buffer[BUFSIZ];
     struct linebuffer *next;
   };
   typedef struct linebuffer LBUFFER;
   LBUFFER *first, *last, *tmp;
-  int i;			/* Index into buffers. */
-  int total_lines = 0;		/* Total number of newlines in all buffers. */
+  int i;			/* Index into buffers.  */
+  int total_lines = 0;		/* Total number of newlines in all buffers.  */
   int errors = 0;
 
   first = last = (LBUFFER *) xmalloc (sizeof (LBUFFER));
@@ -644,13 +674,13 @@ pipe_lines (filename, fd, number)
   first->next = NULL;
   tmp = (LBUFFER *) xmalloc (sizeof (LBUFFER));
 
-  /* Input is always read into a fresh buffer. */
-  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZE)) > 0)
+  /* Input is always read into a fresh buffer.  */
+  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZ)) > 0)
     {
       tmp->nlines = 0;
       tmp->next = NULL;
 
-      /* Count the number of newlines just read. */
+      /* Count the number of newlines just read.  */
       for (i = 0; i < tmp->nbytes; i++)
 	if (tmp->buffer[i] == '\n')
 	  ++tmp->nlines;
@@ -658,10 +688,10 @@ pipe_lines (filename, fd, number)
 
       /* If there is enough room in the last buffer read, just append the new
          one to it.  This is because when reading from a pipe, `nbytes' can
-         often be very small. */
-      if (tmp->nbytes + last->nbytes < BUFSIZE)
+         often be very small.  */
+      if (tmp->nbytes + last->nbytes < BUFSIZ)
 	{
-	  bcopy (tmp->buffer, &last->buffer[last->nbytes], tmp->nbytes);
+	  memcpy (&last->buffer[last->nbytes], tmp->buffer, tmp->nbytes);
 	  last->nbytes += tmp->nbytes;
 	  last->nlines += tmp->nlines;
 	}
@@ -671,9 +701,9 @@ pipe_lines (filename, fd, number)
 	     the list, then either free up the oldest buffer for the next
 	     read if that would leave enough lines, or else malloc a new one.
 	     Some compaction mechanism is possible but probably not
-	     worthwhile. */
+	     worthwhile.  */
 	  last = last->next = tmp;
-	  if (total_lines - first->nlines > number)
+	  if (total_lines - first->nlines > n_lines)
 	    {
 	      tmp = first;
 	      total_lines -= first->nlines;
@@ -693,11 +723,11 @@ pipe_lines (filename, fd, number)
 
   free ((char *) tmp);
 
-  /* This prevents a core dump when the pipe contains no newlines. */
-  if (number == 0)
+  /* This prevents a core dump when the pipe contains no newlines.  */
+  if (n_lines == 0)
     goto free_lbuffers;
 
-  /* Count the incomplete line on files that don't end with a newline. */
+  /* Count the incomplete line on files that don't end with a newline.  */
   if (last->buffer[last->nbytes - 1] != '\n')
     {
       ++last->nlines;
@@ -705,29 +735,29 @@ pipe_lines (filename, fd, number)
     }
 
   /* Run through the list, printing lines.  First, skip over unneeded
-     buffers. */
-  for (tmp = first; total_lines - tmp->nlines > number; tmp = tmp->next)
+     buffers.  */
+  for (tmp = first; total_lines - tmp->nlines > n_lines; tmp = tmp->next)
     total_lines -= tmp->nlines;
 
-  /* Find the correct beginning, then print the rest of the file. */
-  if (total_lines > number)
+  /* Find the correct beginning, then print the rest of the file.  */
+  if (total_lines > n_lines)
     {
       char *cp;
 
-      /* Skip `total_lines' - `number' newlines.  We made sure that
-         `total_lines' - `number' <= `tmp->nlines'. */
+      /* Skip `total_lines' - `n_lines' newlines.  We made sure that
+         `total_lines' - `n_lines' <= `tmp->nlines'.  */
       cp = tmp->buffer;
-      for (i = total_lines - number; i; --i)
+      for (i = total_lines - n_lines; i; --i)
 	while (*cp++ != '\n')
-	  /* Do nothing. */ ;
+	  /* Do nothing.  */ ;
       i = cp - tmp->buffer;
     }
   else
     i = 0;
-  XWRITE (1, &tmp->buffer[i], tmp->nbytes - i);
+  XWRITE (STDOUT_FILENO, &tmp->buffer[i], tmp->nbytes - i);
 
   for (tmp = tmp->next; tmp; tmp = tmp->next)
-    XWRITE (1, tmp->buffer, tmp->nbytes);
+    XWRITE (STDOUT_FILENO, tmp->buffer, tmp->nbytes);
 
 free_lbuffers:
   while (first)
@@ -739,26 +769,26 @@ free_lbuffers:
   return errors;
 }
 
-/* Print the last NUMBER characters from the end of pipe FD.
+/* Print the last N_BYTES characters from the end of pipe FD.
    This is a stripped down version of pipe_lines.
-   Return 0 if successful, 1 if an error occurred. */
+   Return 0 if successful, 1 if an error occurred.  */
 
 static int
-pipe_bytes (filename, fd, number)
-     char *filename;
+pipe_bytes (filename, fd, n_bytes)
+     const char *filename;
      int fd;
-     long number;
+     off_t n_bytes;
 {
   struct charbuffer
   {
     int nbytes;
-    char buffer[BUFSIZE];
+    char buffer[BUFSIZ];
     struct charbuffer *next;
   };
   typedef struct charbuffer CBUFFER;
   CBUFFER *first, *last, *tmp;
-  int i;			/* Index into buffers. */
-  int total_bytes = 0;		/* Total characters in all buffers. */
+  int i;			/* Index into buffers.  */
+  int total_bytes = 0;		/* Total characters in all buffers.  */
   int errors = 0;
 
   first = last = (CBUFFER *) xmalloc (sizeof (CBUFFER));
@@ -766,18 +796,18 @@ pipe_bytes (filename, fd, number)
   first->next = NULL;
   tmp = (CBUFFER *) xmalloc (sizeof (CBUFFER));
 
-  /* Input is always read into a fresh buffer. */
-  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZE)) > 0)
+  /* Input is always read into a fresh buffer.  */
+  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZ)) > 0)
     {
       tmp->next = NULL;
 
       total_bytes += tmp->nbytes;
       /* If there is enough room in the last buffer read, just append the new
          one to it.  This is because when reading from a pipe, `nbytes' can
-         often be very small. */
-      if (tmp->nbytes + last->nbytes < BUFSIZE)
+         often be very small.  */
+      if (tmp->nbytes + last->nbytes < BUFSIZ)
 	{
-	  bcopy (tmp->buffer, &last->buffer[last->nbytes], tmp->nbytes);
+	  memcpy (&last->buffer[last->nbytes], tmp->buffer, tmp->nbytes);
 	  last->nbytes += tmp->nbytes;
 	}
       else
@@ -786,9 +816,9 @@ pipe_bytes (filename, fd, number)
 	     the list, then either free up the oldest buffer for the next
 	     read if that would leave enough characters, or else malloc a new
 	     one.  Some compaction mechanism is possible but probably not
-	     worthwhile. */
+	     worthwhile.  */
 	  last = last->next = tmp;
-	  if (total_bytes - first->nbytes > number)
+	  if (total_bytes - first->nbytes > n_bytes)
 	    {
 	      tmp = first;
 	      total_bytes -= first->nbytes;
@@ -811,20 +841,20 @@ pipe_bytes (filename, fd, number)
   free ((char *) tmp);
 
   /* Run through the list, printing characters.  First, skip over unneeded
-     buffers. */
-  for (tmp = first; total_bytes - tmp->nbytes > number; tmp = tmp->next)
+     buffers.  */
+  for (tmp = first; total_bytes - tmp->nbytes > n_bytes; tmp = tmp->next)
     total_bytes -= tmp->nbytes;
 
   /* Find the correct beginning, then print the rest of the file.
-     We made sure that `total_bytes' - `number' <= `tmp->nbytes'. */
-  if (total_bytes > number)
-    i = total_bytes - number;
+     We made sure that `total_bytes' - `n_bytes' <= `tmp->nbytes'.  */
+  if (total_bytes > n_bytes)
+    i = total_bytes - n_bytes;
   else
     i = 0;
-  XWRITE (1, &tmp->buffer[i], tmp->nbytes - i);
+  XWRITE (STDOUT_FILENO, &tmp->buffer[i], tmp->nbytes - i);
 
   for (tmp = tmp->next; tmp; tmp = tmp->next)
-    XWRITE (1, tmp->buffer, tmp->nbytes);
+    XWRITE (STDOUT_FILENO, tmp->buffer, tmp->nbytes);
 
 free_cbuffers:
   while (first)
@@ -836,50 +866,50 @@ free_cbuffers:
   return errors;
 }
 
-/* Skip NUMBER characters from the start of pipe FD, and print
+/* Skip N_BYTES characters from the start of pipe FD, and print
    any extra characters that were read beyond that.
    Return 1 on error, 0 if ok.  */
 
 static int
-start_bytes (filename, fd, number)
-     char *filename;
+start_bytes (filename, fd, n_bytes)
+     const char *filename;
      int fd;
-     long number;
+     off_t n_bytes;
 {
-  char buffer[BUFSIZE];
+  char buffer[BUFSIZ];
   int bytes_read = 0;
 
-  while (number > 0 && (bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0)
-    number -= bytes_read;
+  while (n_bytes > 0 && (bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0)
+    n_bytes -= bytes_read;
   if (bytes_read == -1)
     {
       error (0, errno, "%s", filename);
       return 1;
     }
-  else if (number < 0)
-    XWRITE (1, &buffer[bytes_read + number], -number);
+  else if (n_bytes < 0)
+    XWRITE (STDOUT_FILENO, &buffer[bytes_read + n_bytes], -n_bytes);
   return 0;
 }
 
-/* Skip NUMBER lines at the start of file or pipe FD, and print
+/* Skip N_LINES lines at the start of file or pipe FD, and print
    any extra characters that were read beyond that.
    Return 1 on error, 0 if ok.  */
 
 static int
-start_lines (filename, fd, number)
-     char *filename;
+start_lines (filename, fd, n_lines)
+     const char *filename;
      int fd;
-     long number;
+     long n_lines;
 {
-  char buffer[BUFSIZE];
+  char buffer[BUFSIZ];
   int bytes_read = 0;
   int bytes_to_skip = 0;
 
-  while (number && (bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0)
+  while (n_lines && (bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0)
     {
       bytes_to_skip = 0;
       while (bytes_to_skip < bytes_read)
-	if (buffer[bytes_to_skip++] == '\n' && --number == 0)
+	if (buffer[bytes_to_skip++] == '\n' && --n_lines == 0)
 	  break;
     }
   if (bytes_read == -1)
@@ -888,7 +918,10 @@ start_lines (filename, fd, number)
       return 1;
     }
   else if (bytes_to_skip < bytes_read)
-    XWRITE (1, &buffer[bytes_to_skip], bytes_read - bytes_to_skip);
+    {
+      XWRITE (STDOUT_FILENO, &buffer[bytes_to_skip],
+	      bytes_read - bytes_to_skip);
+    }
   return 0;
 }
 
@@ -898,24 +931,25 @@ start_lines (filename, fd, number)
 
 static long
 dump_remainder (filename, fd)
-     char *filename;
+     const char *filename;
      int fd;
 {
-  char buffer[BUFSIZE];
+  char buffer[BUFSIZ];
   int bytes_read;
   long total;
 
   total = 0;
 output:
-  while ((bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0)
+  while ((bytes_read = safe_read (fd, buffer, BUFSIZ)) > 0)
     {
-      XWRITE (1, buffer, bytes_read);
+      XWRITE (STDOUT_FILENO, buffer, bytes_read);
       total += bytes_read;
     }
   if (bytes_read == -1)
     error (1, errno, "%s", filename);
   if (forever)
     {
+      fflush (stdout);
       sleep (1);
       goto output;
     }
@@ -966,7 +1000,7 @@ tail_forever (names, nfiles)
 
 	  if (stats.st_size < file_sizes[i])
 	    {
-	      write_header (names[i], "file truncated");
+	      write_header (names[i], _("file truncated"));
 	      last = i;
 	      lseek (file_descs[i], stats.st_size, SEEK_SET);
 	      file_sizes[i] = stats.st_size;
@@ -988,76 +1022,40 @@ tail_forever (names, nfiles)
     }
 }
 
-static void
-parse_unit (str)
-     char *str;
-{
-  int arglen = strlen (str);
-
-  if (arglen == 0)
-    return;
-
-  switch (str[arglen - 1])
-    {
-    case 'b':
-      unit_size = 512;
-      str[arglen - 1] = '\0';
-      break;
-    case 'k':
-      unit_size = 1024;
-      str[arglen - 1] = '\0';
-      break;
-    case 'm':
-      unit_size = 1048576;
-      str[arglen - 1] = '\0';
-      break;
-    }
-}
-
-/* Convert STR, a string of ASCII digits, into an unsigned integer.
-   Return -1 if STR does not represent a valid unsigned integer. */
-
-static long
-atou (str)
-     char *str;
-{
-  unsigned long value;
-
-  for (value = 0; ISDIGIT (*str); ++str)
-    value = value * 10 + *str - '0';
-  return *str ? -1 : value;
-}
-
 static void
 usage (status)
      int status;
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... [FILE]...\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Print last 10 lines of each FILE to standard output.\n\
+With more than one FILE, precede each with a header giving the file name.\n\
+With no FILE, or when FILE is -, read standard input.\n\
 \n\
-  -c, --bytes=SIZE         print last SIZE bytes\n\
-  -f, --follow             print files as they grow\n\
-  -l, -n, --lines=NUMBER   print last NUMBER lines, instead of last 10\n\
-  -q, --quiet, --silent    never print headers giving file names\n\
-  -v, --verbose            always print headers giving file names\n\
+  -c, --bytes=N            output the last N bytes\n\
+  -f, --follow             output appended data as the file grows\n\
+  -n, --lines=N            output the last N lines, instead of last 10\n\
+  -q, --quiet, --silent    never output headers giving file names\n\
+  -v, --verbose            always output headers giving file names\n\
       --help               display this help and exit\n\
       --version            output version information and exit\n\
 \n\
-SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
-If SIZE is prefixed by +, prints all except the first SIZE bytes.  If\n\
-NUMBER is prefixed by +, prints all except the first NUMBER lines.  If\n\
--VALUE or +VALUE is used as first OPTION, read -c VALUE or -c +VALUE\n\
-when one of multipliers bkm follows concatenated, else read -n VALUE\n\
-or -n +VALUE.  With no FILE, or when FILE is -, read standard input.\n\
-");
+If the first character of N (the number of bytes or lines) is a `+',\n\
+print beginning with the Nth item from the start of each file, otherwise,\n\
+print the last N items in the file.  N may have a multiplier suffix:\n\
+b for 512, k for 1024, m for 1048576 (1 Meg).  A first OPTION of -VALUE\n\
+or +VALUE is treated like -n VALUE or -n +VALUE unless VALUE has one of\n\
+the [bkm] suffix multipliers, in which case it is treated like -c VALUE\n\
+or -c +VALUE.\n\
+"));
     }
   exit (status);
 }
