@@ -1,5 +1,5 @@
 /* tr -- a filter to translate characters
-   Copyright (C) 1991 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1995 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by Jim Meyering, meyering@cs.utexas.edu.  */
 
@@ -23,15 +23,22 @@
 #define _GNU_SOURCE
 
 #include <stdio.h>
+#define NDEBUG 1
 #include <assert.h>
 #include <errno.h>
 #include <sys/types.h>
-#include "getopt.h"
+#include <getopt.h>
+
 #include "system.h"
 #include "version.h"
+#include "error.h"
+
+#ifndef ULONG_MAX
+#define ULONG_MAX ((unsigned long) ~(unsigned long) 0)
+#endif
 
 #ifndef LONG_MAX
-#define LONG_MAX 0x7FFFFFFF
+#define LONG_MAX ((long int) (ULONG_MAX >> 1))
 #endif
 
 #ifndef UCHAR_MAX
@@ -138,7 +145,7 @@ struct List_element
 	struct			/* unnamed */
 	  {
 	    unsigned int the_repeated_char;
-	    long repeat_count;
+	    size_t repeat_count;
 	  }
 	repeated_char;
       }
@@ -149,7 +156,7 @@ struct List_element
    to work with: a linked list of constructs (struct List_element).
    Each Spec_list structure also encapsulates various attributes of
    the corresponding argument string.  The attributes are used mainly
-   to verify that the strings are legal in the context of any options
+   to verify that the strings are valid in the context of any options
    specified (like -s, -d, or -c).  The main exception is the member
    `tail', which is first used to construct the list.  After construction,
    it is used by get_next to save its state when traversing the list.
@@ -169,13 +176,18 @@ struct Spec_list
     /* Used to save state between calls to get_next().  */
     unsigned int state;
 
-    /* Length, in the sense that length('a-z[:digit:]123abc')
+    /* Length, in the sense that length ('a-z[:digit:]123abc')
        is 42 ( = 26 + 10 + 6).  */
-    int length;
+    size_t length;
 
     /* The number of [c*] and [c*0] constructs that appear in this spec.  */
     int n_indefinite_repeats;
 
+    /* If n_indefinite_repeats is non-zero, this points to the List_element
+       corresponding to the last [c*] or [c*0] construct encountered in
+       this spec.  Otherwise it is undefined.  */
+    struct List_element *indefinite_repeat_element;
+
     /* Non-zero if this spec contains at least one equivalence
        class construct e.g. [=c=].  */
     int has_equiv_class;
@@ -191,7 +203,6 @@ struct Spec_list
 
 char *xmalloc ();
 char *stpcpy ();
-void error ();
 int safe_read ();
 
 /* The name by which this program was run.  */
@@ -308,19 +319,20 @@ static struct option const long_options[] =
 };
 
 static void
-usage (status)
-     int status;
+usage (int status)
 {
   if (status != 0)
-    fprintf (stderr, "Try `%s --help' for more information.\n",
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
 	     program_name);
   else
     {
-      printf ("\
+      printf (_("\
 Usage: %s [OPTION]... SET1 [SET2]\n\
-",
+"),
 	      program_name);
-      printf ("\
+      printf (_("\
+Translate, squeeze, and/or delete characters from standard input,\n\
+writing to standard output.\n\
 \n\
   -c, --complement        first complement SET1\n\
   -d, --delete            delete characters in SET1, do not translate\n\
@@ -328,11 +340,11 @@ Usage: %s [OPTION]... SET1 [SET2]\n\
   -t, --truncate-set1     first truncate SET1 to length of SET2\n\
       --help              display this help and exit\n\
       --version           output version information and exit\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
-SETs are specified as strings of characters.  Most represent\n\
-themselves.  Here are the special writings:\n\
+SETs are specified as strings of characters.  Most represent themselves.\n\
+Interpreted sequences are:\n\
 \n\
   \\NNN            character with octal value NNN (1 to 3 octal digits)\n\
   \\\\              backslash\n\
@@ -360,8 +372,8 @@ themselves.  Here are the special writings:\n\
   [:upper:]       all upper case letters\n\
   [:xdigit:]      all hexadecimal digits\n\
   [=CHAR=]        all characters which are equivalent to CHAR\n\
-");
-      printf ("\
+"));
+      printf (_("\
 \n\
 Translation occurs if -d is not given and both SET1 and SET2 appear.\n\
 -t may be used only when translating.  SET2 is extended to length of\n\
@@ -371,7 +383,7 @@ expand in ascending order; used in SET2 while translating, they may\n\
 only be used in pairs to specify case conversion.  -s uses SET1 if not\n\
 translating nor deleting; else squeezing uses SET2 and occurs after\n\
 translation or deletion.\n\
-");
+"));
     }
   exit (status);
 }
@@ -380,9 +392,7 @@ translation or deletion.\n\
    equivalence class containing the character EQUIV_CLASS.  */
 
 static int
-is_equiv_class_member (equiv_class, c)
-     unsigned int equiv_class;
-     unsigned int c;
+is_equiv_class_member (unsigned int equiv_class, unsigned int c)
 {
   return (equiv_class == c);
 }
@@ -391,52 +401,53 @@ is_equiv_class_member (equiv_class, c)
    character class CHAR_CLASS.  */
 
 static int
-is_char_class_member (char_class, c)
-     enum Char_class char_class;
-     unsigned int c;
+is_char_class_member (enum Char_class char_class, unsigned int c)
 {
+  int result;
+
   switch (char_class)
     {
     case CC_ALNUM:
-      return ISALNUM (c);
+      result = ISALNUM (c);
       break;
     case CC_ALPHA:
-      return ISALPHA (c);
+      result = ISALPHA (c);
       break;
     case CC_BLANK:
-      return ISBLANK (c);
+      result = ISBLANK (c);
       break;
     case CC_CNTRL:
-      return ISCNTRL (c);
+      result = ISCNTRL (c);
       break;
     case CC_DIGIT:
-      return ISDIGIT (c);
+      result = ISDIGIT (c);
       break;
     case CC_GRAPH:
-      return ISGRAPH (c);
+      result = ISGRAPH (c);
       break;
     case CC_LOWER:
-      return ISLOWER (c);
+      result = ISLOWER (c);
       break;
     case CC_PRINT:
-      return ISPRINT (c);
+      result = ISPRINT (c);
       break;
     case CC_PUNCT:
-      return ISPUNCT (c);
+      result = ISPUNCT (c);
       break;
     case CC_SPACE:
-      return ISSPACE (c);
+      result = ISSPACE (c);
       break;
     case CC_UPPER:
-      return ISUPPER (c);
+      result = ISUPPER (c);
       break;
     case CC_XDIGIT:
-      return ISXDIGIT (c);
+      result = ISXDIGIT (c);
       break;
     default:
       abort ();
       break;
     }
+  return result;
 }
 
 /* Perform the first pass over each range-spec argument S, converting all
@@ -449,11 +460,9 @@ is_char_class_member (char_class, c)
    cannot contain actual (non-escaped) zero bytes.  */
 
 static int
-unquote (s, len)
-     unsigned char *s;
-     int *len;
+unquote (unsigned char *s, size_t *len)
 {
-  int i, j;
+  size_t i, j;
 
   j = 0;
   for (i = 0; s[i]; i++)
@@ -520,8 +529,8 @@ unquote (s, len)
 			     isn't clear on this, but one person has said
 			     that in his interpretation, POSIX says tr
 			     can't even give a warning.  */
-			  error (0, 0, "warning: the ambiguous octal escape \
-\\%c%c%c is being\n\tinterpreted as the 2-byte sequence \\0%c%c, `%c'",
+			  error (0, 0, _("warning: the ambiguous octal escape \
+\\%c%c%c is being\n\tinterpreted as the 2-byte sequence \\0%c%c, `%c'"),
 				 s[i], s[i + 1], s[i + 2],
 				 s[i], s[i + 1], s[i + 2]);
 			}
@@ -529,13 +538,12 @@ unquote (s, len)
 		}
 	      break;
 	    case '\0':
-	      error (0, 0, "invalid backslash escape at end of string");
+	      error (0, 0, _("invalid backslash escape at end of string"));
 	      return 1;
-	      break;
+
 	    default:
-	      error (0, 0, "invalid backslash escape `\\%c'", s[i + 1]);
+	      error (0, 0, _("invalid backslash escape `\\%c'"), s[i + 1]);
 	      return 1;
-	      break;
 	    }
 	  ++i;
 	  s[j++] = c;
@@ -553,13 +561,13 @@ unquote (s, len)
    in the global char_class_name array.  Otherwise, return CC_NO_CLASS.  */
 
 static enum Char_class
-look_up_char_class (class_str)
-     unsigned char *class_str;
+look_up_char_class (const unsigned char *class_str, size_t len)
 {
   unsigned int i;
 
   for (i = 0; i < N_CHAR_CLASSES; i++)
-    if (strcmp ((const char *) class_str, char_class_name[i]) == 0)
+    if (strncmp ((const char *) class_str, char_class_name[i], len) == 0
+	&& strlen (char_class_name[i]) == len)
       return (enum Char_class) i;
   return CC_NO_CLASS;
 }
@@ -568,8 +576,7 @@ look_up_char_class (class_str)
    This function is used solely for formatting error messages.  */
 
 static char *
-make_printable_char (c)
-     unsigned int c;
+make_printable_char (unsigned int c)
 {
   char *buf = xmalloc (5);
 
@@ -594,15 +601,13 @@ make_printable_char (c)
    sequences.  This function is used solely for printing error messages.  */
 
 static char *
-make_printable_str (s, len)
-     const unsigned char *s;
-     int len;
+make_printable_str (const unsigned char *s, size_t len)
 {
   /* Worst case is that every character expands to a backslash
      followed by a 3-character octal escape sequence.  */
   char *printable_buf = xmalloc (4 * len + 1);
   char *p = printable_buf;
-  int i;
+  size_t i;
 
   for (i = 0; i < len; i++)
     {
@@ -655,9 +660,7 @@ make_printable_str (s, len)
    character C to the specification list LIST.  */
 
 static void
-append_normal_char (list, c)
-     struct Spec_list *list;
-     unsigned int c;
+append_normal_char (struct Spec_list *list, unsigned int c)
 {
   struct List_element *new;
 
@@ -676,10 +679,7 @@ append_normal_char (list, c)
    zero otherwise.  This means that '[c-c]' is acceptable.  */
 
 static int
-append_range (list, first, last)
-     struct Spec_list *list;
-     unsigned int first;
-     unsigned int last;
+append_range (struct Spec_list *list, unsigned int first, unsigned int last)
 {
   struct List_element *new;
 
@@ -689,7 +689,7 @@ append_range (list, first, last)
       char *tmp2 = make_printable_char (last);
 
       error (0, 0,
-       "range-endpoints of `%s-%s' are in reverse collating sequence order",
+       _("range-endpoints of `%s-%s' are in reverse collating sequence order"),
 	     tmp1, tmp2);
       free (tmp1);
       free (tmp2);
@@ -712,20 +712,17 @@ append_range (list, first, last)
    a valid string, print an error message and return non-zero.  */
 
 static int
-append_char_class (list, char_class_str, len)
-     struct Spec_list *list;
-     unsigned char *char_class_str;
-     int len;
+append_char_class (struct Spec_list *list, const unsigned char *char_class_str, size_t len)
 {
   enum Char_class char_class;
   struct List_element *new;
 
-  char_class = look_up_char_class (char_class_str);
+  char_class = look_up_char_class (char_class_str, len);
   if (char_class == CC_NO_CLASS)
     {
       char *tmp = make_printable_str (char_class_str, len);
 
-      error (0, 0, "invalid character class `%s'", tmp);
+      error (0, 0, _("invalid character class `%s'"), tmp);
       free (tmp);
       return 1;
     }
@@ -745,10 +742,7 @@ append_char_class (list, char_class_str, len)
    is a non-negative repeat count.  */
 
 static void
-append_repeated_char (list, the_char, repeat_count)
-     struct Spec_list *list;
-     unsigned int the_char;
-     long int repeat_count;
+append_repeated_char (struct Spec_list *list, unsigned int the_char, size_t repeat_count)
 {
   struct List_element *new;
 
@@ -769,10 +763,7 @@ append_repeated_char (list, the_char, repeat_count)
    If LEN is not 1, issue an error message and return non-zero.  */
 
 static int
-append_equiv_class (list, equiv_class_str, len)
-     struct Spec_list *list;
-     unsigned char *equiv_class_str;
-     int len;
+append_equiv_class (struct Spec_list *list, const unsigned char *equiv_class_str, size_t len)
 {
   struct List_element *new;
 
@@ -780,7 +771,7 @@ append_equiv_class (list, equiv_class_str, len)
     {
       char *tmp = make_printable_str (equiv_class_str, len);
 
-      error (0, 0, "%s: equivalence class operand must be a single character",
+      error (0, 0, _("%s: equivalence class operand must be a single character"),
 	     tmp);
       free (tmp);
       return 1;
@@ -795,43 +786,44 @@ append_equiv_class (list, equiv_class_str, len)
   return 0;
 }
 
-/* Return a newly allocated copy of the substring P[FIRST_IDX..LAST_IDX].  */
+/* Return a newly allocated copy of the substring P[FIRST_IDX..LAST_IDX].
+   The returned string has length LAST_IDX - FIRST_IDX + 1, may contain
+   NUL bytes, and is *not* NUL-terminated.  */
 
 static unsigned char *
-substr (p, first_idx, last_idx)
-     const unsigned char *p;
-     int first_idx;
-     int last_idx;
+substr (const unsigned char *p, size_t first_idx, size_t last_idx)
 {
-  int len = last_idx - first_idx + 1;
-  unsigned char *tmp = (unsigned char *) xmalloc (len + 1);
+  size_t len;
+  unsigned char *tmp;
 
   assert (first_idx <= last_idx);
-  /* We must use bcopy or memcpy rather than strncpy
-     because `p' may contain zero-bytes.  */
-  bcopy (p + first_idx, tmp, len);
-  tmp[len] = '\0';
+  len = last_idx - first_idx + 1;
+  tmp = (unsigned char *) xmalloc (len);
+
+  assert (first_idx <= last_idx);
+  /* Use memcpy rather than strncpy because `p' may contain zero-bytes.  */
+  memcpy (tmp, p + first_idx, len);
   return tmp;
 }
 
 /* Search forward starting at START_IDX for the 2-char sequence
    (PRE_BRACKET_CHAR,']') in the string P of length P_LEN.  If such
-   a sequence is found, return the index of the first character,
-   otherwise return -1.  P may contain zero bytes.  */
+   a sequence is found, set *RESULT_IDX to the index of the first
+   character and return non-zero. Otherwise return zero.  P may contain
+   zero bytes.  */
 
 static int
-find_closing_delim (p, start_idx, p_len, pre_bracket_char)
-     const unsigned char *p;
-     int start_idx;
-     int p_len;
-     unsigned int pre_bracket_char;
+find_closing_delim (const unsigned char *p, size_t start_idx, size_t p_len, unsigned int pre_bracket_char, size_t *result_idx)
 {
-  int i;
+  size_t i;
 
   for (i = start_idx; i < p_len - 1; i++)
     if (p[i] == pre_bracket_char && p[i + 1] == ']')
-      return i;
-  return -1;
+      {
+	*result_idx = i;
+	return 1;
+      }
+  return 0;
 }
 
 /* Convert a string S with explicit length LEN, possibly
@@ -842,13 +834,10 @@ find_closing_delim (p, start_idx, p_len, pre_bracket_char)
    Otherwise, return zero and set *VAL to the converted value.  */
 
 static int
-non_neg_strtol (s, len, val)
-     const unsigned char *s;
-     int len;
-     long int *val;
+non_neg_strtol (const unsigned char *s, size_t len, size_t *val)
 {
-  int i;
-  long sum = 0;
+  size_t i;
+  unsigned long sum = 0;
   unsigned int base;
 
   if (len <= 0)
@@ -862,11 +851,16 @@ non_neg_strtol (s, len, val)
 
   for (i = 0; i < len; i++)
     {
-      int c = s[i] - '0';
+      unsigned int c;
 
-      if (c >= base || c < 0)
+      if (s[i] < '0')
 	return 1;
-      if (i > 8 && sum > (LONG_MAX - c) / base)
+
+      c = s[i] - '0';
+      if (c >= base)
+	return 1;
+
+      if (sum > (LONG_MAX - c) / base)
 	return 1;
       sum = sum * base + c;
     }
@@ -876,9 +870,8 @@ non_neg_strtol (s, len, val)
 
 /* Parse the bracketed repeat-char syntax.  If the P_LEN characters
    beginning with P[ START_IDX ] comprise a valid [c*n] construct,
-   return the character and the repeat count through the arg pointers,
-   CHAR_TO_REPEAT and N, and then return the index of the closing
-   bracket as the function value.  If the second character following
+   then set *CHAR_TO_REPEAT, *REPEAT_COUNT, and *CLOSING_BRACKET_IDX
+   and return zero. If the second character following
    the opening bracket is not `*' or if no closing bracket can be
    found, return -1.  If a closing bracket is found and the
    second char is `*', but the string between the `*' and `]' isn't
@@ -886,14 +879,9 @@ non_neg_strtol (s, len, val)
    and return -2.  */
 
 static int
-find_bracketed_repeat (p, start_idx, p_len, char_to_repeat, n)
-     const unsigned char *p;
-     int start_idx;
-     int p_len;
-     unsigned int *char_to_repeat;
-     long int *n;
+find_bracketed_repeat (const unsigned char *p, size_t start_idx, size_t p_len, unsigned int *char_to_repeat, size_t *repeat_count, size_t *closing_bracket_idx)
 {
-  int i;
+  size_t i;
 
   assert (start_idx + 1 < p_len);
   if (p[start_idx + 1] != '*')
@@ -904,27 +892,30 @@ find_bracketed_repeat (p, start_idx, p_len, char_to_repeat, n)
       if (p[i] == ']')
 	{
 	  const unsigned char *digit_str;
-	  int digit_str_len = i - start_idx - 2;
+	  size_t digit_str_len = i - start_idx - 2;
 
 	  *char_to_repeat = p[start_idx];
 	  if (digit_str_len == 0)
 	    {
 	      /* We've matched [c*] -- no explicit repeat count.  */
-	      *n = 0;
-	      return i;
+	      *repeat_count = 0;
+	      *closing_bracket_idx = i;
+	      return 0;
 	    }
 
 	  /* Here, we have found [c*s] where s should be a string
 	     of octal or decimal digits.  */
 	  digit_str = &p[start_idx + 2];
-	  if (non_neg_strtol (digit_str, digit_str_len, n))
+	  if (non_neg_strtol (digit_str, digit_str_len, repeat_count))
 	    {
 	      char *tmp = make_printable_str (digit_str, digit_str_len);
-	      error (0, 0, "invalid repeat count `%s' in [c*n] construct", tmp);
+	      error (0, 0, _("invalid repeat count `%s' in [c*n] construct"),
+		     tmp);
 	      free (tmp);
 	      return -2;
 	    }
-	  return i;
+	  *closing_bracket_idx = i;
+	  return 0;
 	}
     }
   return -1;			/* No bracket found.  */
@@ -943,13 +934,10 @@ find_bracketed_repeat (p, start_idx, p_len, char_to_repeat, n)
       - c Any other character is interpreted as itself.  */
 
 static int
-build_spec_list (unescaped_string, len, result)
-     const unsigned char *unescaped_string;
-     int len;
-     struct Spec_list *result;
+build_spec_list (const unsigned char *unescaped_string, size_t len, struct Spec_list *result)
 {
   const unsigned char *p;
-  int i;
+  size_t i;
 
   p = unescaped_string;
 
@@ -960,23 +948,28 @@ build_spec_list (unescaped_string, len, result)
      less are composed solely of normal characters.  Hence, the index of
      the outer for-loop runs only as far as LEN-2.  */
 
-  for (i = 0; i < len - 2;)
+  for (i = 0; i + 2 < len;)
     {
       switch (p[i])
 	{
 	  int fall_through;
+	  int err;
+
 	case '[':
 	  fall_through = 0;
 	  switch (p[i + 1])
 	    {
-	      int closing_delim_idx;
-	      int closing_bracket_idx;
+	      size_t closing_delim_idx;
+	      size_t closing_bracket_idx;
 	      unsigned int char_to_repeat;
-	      long repeat_count;
+	      size_t repeat_count;
+	      int found;
+
 	    case ':':
 	    case '=':
-	      closing_delim_idx = find_closing_delim (p, i + 2, len, p[i + 1]);
-	      if (closing_delim_idx >= 0)
+	      found = find_closing_delim (p, i + 2, len, p[i + 1],
+					  &closing_delim_idx);
+	      if (found)
 		{
 		  int parse_failed;
 		  unsigned char *opnd_str = substr (p, i + 2,
@@ -1000,15 +993,16 @@ build_spec_list (unescaped_string, len, result)
 	    default:
 	      /* Determine whether this is a bracketed repeat range
 	         matching the RE \[.\*(dec_or_oct_number)?\].  */
-	      closing_bracket_idx = find_bracketed_repeat (p, i + 1,
-				       len, &char_to_repeat, &repeat_count);
-	      if (closing_bracket_idx >= 0)
+	      err = find_bracketed_repeat (p, i + 1, len, &char_to_repeat,
+					   &repeat_count,
+					   &closing_bracket_idx);
+	      if (err == 0)
 		{
 		  append_repeated_char (result, char_to_repeat, repeat_count);
 		  i = closing_bracket_idx + 1;
 		  break;
 		}
-	      else if (closing_bracket_idx == -1)
+	      else if (err == -1)
 		{
 		  fall_through = 1;
 		}
@@ -1063,9 +1057,7 @@ build_spec_list (unescaped_string, len, result)
    positions.  */
 
 static int
-get_next (s, class)
-     struct Spec_list *s;
-     enum Upper_Lower_class *class;
+get_next (struct Spec_list *s, enum Upper_Lower_class *class)
 {
   struct List_element *p;
   int return_val;
@@ -1156,7 +1148,6 @@ get_next (s, class)
 
     case RE_REPEATED_CHAR:
       /* Here, a repeat count of n == 0 means don't repeat at all.  */
-      assert (p->u.repeated_char.repeat_count >= 0);
       if (p->u.repeated_char.repeat_count == 0)
 	{
 	  s->tail = p->next;
@@ -1198,14 +1189,13 @@ get_next (s, class)
    same operations are (duplicated) performed in set_initialize.  */
 
 static int
-card_of_complement (s)
-     struct Spec_list *s;
+card_of_complement (struct Spec_list *s)
 {
   int c;
   int cardinality = N_CHARS;
   SET_TYPE in_set[N_CHARS];
 
-  bzero (in_set, N_CHARS * sizeof (in_set[0]));
+  memset (in_set, 0, N_CHARS * sizeof (in_set[0]));
   s->state = BEGIN_STATE;
   while ((c = get_next (s, NULL)) != -1)
     if (!in_set[c]++)
@@ -1214,7 +1204,7 @@ card_of_complement (s)
 }
 
 /* Gather statistics about the spec-list S in preparation for the tests
-   in validate that determine the legality of the specs.  This function
+   in validate that determine the consistency of the specs.  This function
    is called at most twice; once for string1, and again for any string2.
    LEN_S1 < 0 indicates that this is the first call and that S represents
    string1.  When LEN_S1 >= 0, it is the length of the expansion of the
@@ -1227,12 +1217,9 @@ card_of_complement (s)
    be 26 and S (representing string2) would be converted to 'A[\n*24]Z'.  */
 
 static void
-get_spec_stats (s, len_s1)
-     struct Spec_list *s;
-     int len_s1;
+get_spec_stats (struct Spec_list *s)
 {
   struct List_element *p;
-  struct List_element *indefinite_repeat_element = NULL;
   int len = 0;
 
   s->n_indefinite_repeats = 0;
@@ -1281,7 +1268,7 @@ get_spec_stats (s, len_s1)
 	    len += p->u.repeated_char.repeat_count;
 	  else if (p->u.repeated_char.repeat_count == 0)
 	    {
-	      indefinite_repeat_element = p;
+	      s->indefinite_repeat_element = p;
 	      ++(s->n_indefinite_repeats);
 	    }
 	  break;
@@ -1292,21 +1279,31 @@ get_spec_stats (s, len_s1)
 	}
     }
 
-  if (len_s1 >= len && s->n_indefinite_repeats == 1)
-    {
-      indefinite_repeat_element->u.repeated_char.repeat_count = len_s1 - len;
-      len = len_s1;
-    }
-  if (complement && len_s1 < 0)
-    s->length = card_of_complement (s);
-  else
-    s->length = len;
-  return;
+  s->length = len;
 }
 
 static void
-spec_init (spec_list)
-     struct Spec_list *spec_list;
+get_s1_spec_stats (struct Spec_list *s1)
+{
+  get_spec_stats (s1);
+  if (complement)
+    s1->length = card_of_complement (s1);
+}
+
+static void
+get_s2_spec_stats (struct Spec_list *s2, size_t len_s1)
+{
+  get_spec_stats (s2);
+  if (len_s1 >= s2->length && s2->n_indefinite_repeats == 1)
+    {
+      s2->indefinite_repeat_element->u.repeated_char.repeat_count =
+	len_s1 - s2->length;
+      s2->length = len_s1;
+    }
+}
+
+static void
+spec_init (struct Spec_list *spec_list)
 {
   spec_list->head = spec_list->tail =
     (struct List_element *) xmalloc (sizeof (struct List_element));
@@ -1320,11 +1317,9 @@ spec_init (spec_list)
    of these passes detects an error, this function returns non-zero.  */
 
 static int
-parse_str (s, spec_list)
-     unsigned char *s;
-     struct Spec_list *spec_list;
+parse_str (unsigned char *s, struct Spec_list *spec_list)
 {
-  int len;
+  size_t len;
 
   if (unquote (s, &len))
     return 1;
@@ -1349,9 +1344,7 @@ parse_str (s, spec_list)
    sequence.  */
 
 static void
-string2_extend (s1, s2)
-     const struct Spec_list *s1;
-     struct Spec_list *s2;
+string2_extend (const struct Spec_list *s1, struct Spec_list *s2)
 {
   struct List_element *p;
   int char_to_repeat;
@@ -1399,26 +1392,23 @@ string2_extend (s1, s2)
 
   append_repeated_char (s2, char_to_repeat, s1->length - s2->length);
   s2->length = s1->length;
-  return;
 }
 
 /* Die with an error message if S1 and S2 describe strings that
    are not valid with the given command line switches.
-   A side effect of this function is that if a legal [c*] or
+   A side effect of this function is that if a valid [c*] or
    [c*0] construct appears in string2, it is converted to [c*n]
    with a value for n that makes s2->length == s1->length.  By
    the same token, if the --truncate-set1 option is not
    given, S2 may be extended.  */
 
 static void
-validate (s1, s2)
-     const struct Spec_list *s1;
-     struct Spec_list *s2;
+validate (struct Spec_list *s1, struct Spec_list *s2)
 {
-  get_spec_stats (s1, -1);
+  get_s1_spec_stats (s1);
   if (s1->n_indefinite_repeats > 0)
     {
-      error (1, 0, "the [c*] repeat construct may not appear in string1");
+      error (1, 0, _("the [c*] repeat construct may not appear in string1"));
     }
 
   /* FIXME: it isn't clear from the POSIX spec that this is invalid,
@@ -1427,23 +1417,23 @@ validate (s1, s2)
   if (complement && s1->has_upper_or_lower)
     {
       error (1, 0,
-	     "character classes may not be used when translating \
-and complementing");
+	     _("character classes may not be used when translating \
+and complementing"));
     }
 
   if (s2)
     {
-      get_spec_stats (s2, s1->length);
+      get_s2_spec_stats (s2, s1->length);
       if (s2->has_restricted_char_class)
 	{
 	  error (1, 0,
-		 "when translating, the only character classes that may \
-appear in\n\tstring2 are `upper' and `lower'");
+		 _("when translating, the only character classes that may \
+appear in\n\tstring2 are `upper' and `lower'"));
 	}
 
       if (s2->n_indefinite_repeats > 1)
 	{
-	  error (1, 0, "only one [c*] repeat construct may appear in string2");
+	  error (1, 0, _("only one [c*] repeat construct may appear in string2"));
 	}
 
       if (translating)
@@ -1451,8 +1441,8 @@ appear in\n\tstring2 are `upper' and `lower'");
 	  if (s2->has_equiv_class)
 	    {
 	      error (1, 0,
-		     "[=c=] expressions may not appear in string2 \
-when translating");
+		     _("[=c=] expressions may not appear in string2 \
+when translating"));
 	    }
 
 	  if (s1->length > s2->length)
@@ -1464,23 +1454,23 @@ when translating");
 
 		  if (s2->length == 0)
 		    error (1, 0,
-		     "when not truncating set1, string2 must be non-empty");
+		     _("when not truncating set1, string2 must be non-empty"));
 		  string2_extend (s1, s2);
 		}
 	    }
 
 	  if (complement && s2->has_upper_or_lower)
 	    error (1, 0,
-		   "character classes may not be used when translating \
-and complementing");
+		   _("character classes may not be used when translating \
+and complementing"));
 	}
       else
 	/* Not translating.  */
 	{
 	  if (s2->n_indefinite_repeats > 0)
 	    error (1, 0,
-		   "the [c*] construct may appear in string2 only \
-when translating");
+		   _("the [c*] construct may appear in string2 only \
+when translating"));
 	}
     }
 }
@@ -1494,10 +1484,7 @@ when translating");
    character is in the squeeze set.  */
 
 static void
-squeeze_filter (buf, size, reader)
-     unsigned char *buf;
-     long int size;
-     PFI reader;
+squeeze_filter (unsigned char *buf, long int size, PFI reader)
 {
   unsigned int char_to_squeeze = NOT_A_CHAR;
   int i = 0;
@@ -1515,7 +1502,7 @@ squeeze_filter (buf, size, reader)
 	    nr = (*reader) (buf, size, NULL);
 
 	  if (nr < 0)
-	    error (1, errno, "read error");
+	    error (1, errno, _("read error"));
 	  if (nr == 0)
 	    break;
 	  i = 0;
@@ -1566,7 +1553,7 @@ squeeze_filter (buf, size, reader)
 	    }
 	  if (out_len > 0
 	      && fwrite ((char *) &buf[begin], 1, out_len, stdout) == 0)
-	    error (1, errno, "write error");
+	    error (1, errno, _("write error"));
 	}
 
       if (char_to_squeeze != NOT_A_CHAR)
@@ -1592,10 +1579,7 @@ squeeze_filter (buf, size, reader)
    or 0 upon EOF.  */
 
 static long
-read_and_delete (buf, size, not_used)
-     unsigned char *buf;
-     long int size;
-     PFI not_used;
+read_and_delete (unsigned char *buf, long int size, PFI not_used)
 {
   long n_saved;
   static int hit_eof = 0;
@@ -1615,7 +1599,7 @@ read_and_delete (buf, size, not_used)
       int nr = safe_read (0, (char *) buf, size);
 
       if (nr < 0)
-	error (1, errno, "read error");
+	error (1, errno, _("read error"));
       if (nr == 0)
 	{
 	  hit_eof = 1;
@@ -1645,10 +1629,7 @@ read_and_delete (buf, size, not_used)
    array `xlate'.  Return the number of characters read, or 0 upon EOF.  */
 
 static long
-read_and_xlate (buf, size, not_used)
-     unsigned char *buf;
-     long int size;
-     PFI not_used;
+read_and_xlate (unsigned char *buf, long int size, PFI not_used)
 {
   long chars_read = 0;
   static int hit_eof = 0;
@@ -1662,7 +1643,7 @@ read_and_xlate (buf, size, not_used)
 
   chars_read = safe_read (0, (char *) buf, size);
   if (chars_read < 0)
-    error (1, errno, "read error");
+    error (1, errno, _("read error"));
   if (chars_read == 0)
     {
       hit_eof = 1;
@@ -1681,15 +1662,12 @@ read_and_xlate (buf, size, not_used)
    non-zero the resulting set is complemented.  */
 
 static void
-set_initialize (s, complement_this_set, in_set)
-     struct Spec_list *s;
-     int complement_this_set;
-     SET_TYPE *in_set;
+set_initialize (struct Spec_list *s, int complement_this_set, SET_TYPE *in_set)
 {
   int c;
   int i;
 
-  bzero (in_set, N_CHARS * sizeof (in_set[0]));
+  memset (in_set, 0, N_CHARS * sizeof (in_set[0]));
   s->state = BEGIN_STATE;
   while ((c = get_next (s, NULL)) != -1)
     in_set[c] = 1;
@@ -1699,9 +1677,7 @@ set_initialize (s, complement_this_set, in_set)
 }
 
 void
-main (argc, argv)
-     int argc;
-     char **argv;
+main (int argc, char **argv)
 {
   int c;
   int non_option_args;
@@ -1755,23 +1731,23 @@ main (argc, argv)
   non_option_args = argc - optind;
   translating = (non_option_args == 2 && !delete);
 
-  /* Change this test if it is legal to give tr no options and
+  /* Change this test if it is valid to give tr no options and
      no args at all.  POSIX doesn't specifically say anything
      either way, but it looks like they implied it's invalid
      by omission.  If you want to make tr do a slow imitation
      of `cat' use `tr a a'.  */
   if (non_option_args > 2)
     {
-      error (0, 0, "too many arguments");
+      error (0, 0, _("too many arguments"));
       usage (2);
     }
 
   if (!delete && !squeeze_repeats && non_option_args != 2)
-    error (1, 0, "two strings must be given when translating");
+    error (1, 0, _("two strings must be given when translating"));
 
   if (delete && squeeze_repeats && non_option_args != 2)
-    error (1, 0, "two strings must be given when both \
-deleting and squeezing repeats");
+    error (1, 0, _("two strings must be given when both \
+deleting and squeezing repeats"));
 
   /* If --delete is given without --squeeze-repeats, then
      only one string argument may be specified.  But POSIX
@@ -1784,12 +1760,12 @@ deleting and squeezing repeats");
 	--non_option_args;
       else
 	error (1, 0,
-	       "only one string may be given when deleting \
-without squeezing repeats");
+	       _("only one string may be given when deleting \
+without squeezing repeats"));
     }
 
   if (squeeze_repeats && non_option_args == 0)
-    error (1, 0, "at least one string must be given when squeezing repeats");
+    error (1, 0, _("at least one string must be given when squeezing repeats"));
 
   spec_init (s1);
   if (parse_str ((unsigned char *) argv[optind], s1))
@@ -1813,14 +1789,14 @@ without squeezing repeats");
     }
   else if (delete && non_option_args == 1)
     {
-      int nr;
+      long nr;
 
       set_initialize (s1, complement, in_delete_set);
       do
 	{
 	  nr = read_and_delete (io_buf, IO_BUF_SIZE, NULL);
 	  if (nr > 0 && fwrite ((char *) io_buf, 1, nr, stdout) == 0)
-	    error (1, errno, "write error");
+	    error (1, errno, _("write error"));
 	}
       while (nr > 0);
     }
@@ -1875,7 +1851,7 @@ without squeezing repeats");
 	      c2 = get_next (s2, &class_s2);
 	      if (!class_ok[(int) class_s1][(int) class_s2])
 		error (1, 0,
-		     "misaligned or mismatched upper and/or lower classes");
+		     _("misaligned or mismatched upper and/or lower classes"));
 	      /* The following should have been checked by validate...  */
 	      if (c2 == -1)
 		break;
@@ -1890,24 +1866,24 @@ without squeezing repeats");
 	}
       else
 	{
-	  int chars_read;
+	  long chars_read;
 
 	  do
 	    {
 	      chars_read = read_and_xlate (io_buf, IO_BUF_SIZE, NULL);
 	      if (chars_read > 0
 		  && fwrite ((char *) io_buf, 1, chars_read, stdout) == 0)
-		error (1, errno, "write error");
+		error (1, errno, _("write error"));
 	    }
 	  while (chars_read > 0);
 	}
     }
 
   if (fclose (stdout) == EOF)
-    error (2, errno, "write error");
+    error (2, errno, _("write error"));
 
   if (close (0) != 0)
-    error (2, errno, "standard input");
+    error (2, errno, _("standard input"));
 
   exit (0);
 }
